import glob
import os
from scipy import ndimage
import tensorflow as tf
from Label import class2label


def inputPipeline(filenames,
                  batch_size,
                  num_epochs,
                  num_threads,
                  preprocess_type,
                  old_shape,
                  is_3d=False,
                  resize_type=0,
                  new_shape=None):
    filename_queue = tf.train.string_input_producer(filenames, num_epochs=num_epochs, shuffle=True)
    example, label = _readFrom(filename_queue,
                               preprocess_type,
                               old_shape,
                               resize_type,
                               new_shape,
                               is_3d=is_3d)

    #   min_after_dequeue defines how big a buffer we will randomly sample
    #   bigger means better shuffling but slower start up and more
    #   memory used.
    #   capacity must be larger than min_after_dequeue and the amount larger
    #   determines the maximum we will prefetch.  Recommendation:
    #   min_after_dequeue + (num_threads + a small safety margin) * batch_size
    min_after_dequeue = 1000
    capacity = min_after_dequeue + (num_threads+3) * batch_size
    example_batch, label_batch = tf.train.shuffle_batch(
        [example, label],
        batch_size=batch_size,
        num_threads=num_threads,
        capacity=capacity, # Ensures a minimum amount of shuffling
        min_after_dequeue=min_after_dequeue,
        name='batching_shuffing')

    return example_batch, label_batch

def _readFrom(filename_queue,
              preprocess_type,
              old_shape,
              resize_type,
              new_shape,
              is_3d):

    reader = tf.TFRecordReader()
    key, serialized_example = reader.read(filename_queue)

    features = tf.parse_single_example(
        serialized_example,
        features={'image_raw': tf.FixedLenFeature([], tf.string),
                  'label': tf.FixedLenFeature([], tf.int64)})

    image = tf.decode_raw(features['image_raw'], tf.uint8)
    image = tf.cast(image, tf.float32)
    label = tf.cast(features['label'], tf.int32)
    image = tf.reshape(image, old_shape)
    print('original image shape: ', image.get_shape())

    if not is_3d:
        if resize_type == 1:
            # do nothing
            image.set_shape(old_shape)
        elif resize_type == 2:
            # random flip
            image = tf.image.random_flip_up_down(image)
            image = tf.image.random_flip_left_right(image)
            image.set_shape(old_shape)
        elif resize_type == 3:
            # crop (0,0)
            image = tf.image.crop_to_bounding_box(image, 0, 0, new_shape[0], new_shape[1])
            image = tf.reshape(image, new_shape)
            image.set_shape(new_shape)
        elif resize_type == 4:
            # random crop
            image = tf.random_crop(image, [new_shape[0], new_shape[1], new_shape[2]])
            image = tf.reshape(image, new_shape)
            image.set_shape(new_shape)
        elif resize_type == 5:
            # random crop and random flip
            image = tf.random_crop(image, [new_shape[0], new_shape[1], new_shape[2]])
            image = tf.image.random_flip_up_down(image)
            image = tf.image.random_flip_left_right(image)
            image = tf.reshape(image, new_shape)
            image.set_shape(new_shape)
        elif resize_type == 6:
            # random crop and random flip
            image = tf.image.resize_images(image, [new_shape[0], new_shape[1]])
            image = tf.image.random_flip_up_down(image)
            image = tf.image.random_flip_left_right(image)
            image = tf.reshape(image, new_shape)
            image.set_shape(new_shape)

        if preprocess_type>0:
            image = tf.image.random_brightness(image,max_delta=63)
            image = tf.image.random_contrast(image,lower=0.2, upper=1.8)
    else:
        if resize_type == 1:  # Just resize
            # do nothing
            image.set_shape(old_shape)
        elif resize_type == 2:
            # random swap axes
            swap_axises = tf.constant([0,1,2])
            swap_axises = tf.random_shuffle(swap_axises)
            image = tf.transpose(image, perm=[swap_axises[0],
                                              swap_axises[1],
                                              swap_axises[2],
                                              3])
            image.set_shape(old_shape)
        elif resize_type == 3:
            ## Random crop
            image = tf.random_crop(image, tuple(new_shape))
            image = tf.reshape(image, new_shape)
            image.set_shape(new_shape)
        elif resize_type == 4:
            ## Random crop, flip
            image = tf.random_crop(image, tuple(new_shape))
            swap_axises = tf.constant([0,1,2])
            swap_axises = tf.random_shuffle(swap_axises)
            image = tf.transpose(image, perm=[swap_axises[0],
                                              swap_axises[1],
                                              swap_axises[2],
                                              3])
            # noise = tf.random_uniform(tuple(new_shape),minval=0,maxval=None,dtype=tf.float32)
            image = tf.reshape(image, new_shape)
            image.set_shape(new_shape)
        elif resize_type == 5:
            ## central crop
            image = tf.slice(image, [4,4,4,0], new_shape) 
            swap_axises = tf.constant([0,1,2])
            swap_axises = tf.random_shuffle(swap_axises)
            image = tf.transpose(image, perm=[swap_axises[0],
                                              swap_axises[1],
                                              swap_axises[2],
                                              3])
            image.set_shape(new_shape)


        if preprocess_type>0:
            image = tf.image.random_contrast(image,lower=0.3, upper=1.5)


    print('new image shape: ', image.get_shape())
    print('new label shape: ', label.get_shape())

    return image, label


def preprocess(array):
    # swap_axises = tf.constant([0,1,2])
    # c = tf.random_shuffle(c)
    return ndimage.rotate(array, 45)



import unittest
import random
import math
from pyneval.model.euclidean_point import EuclideanPoint,Line


def rand(k):
    return random.uniform(0, k)


class TestPointMethods(unittest.TestCase):

    def test_point_to_line(self):
        p = EuclideanPoint([49.4362, 111.12, 322.687])
        l = Line(coords=[[47.9082, 110.024, 323.994],[56.0636, 112.369, 318.703]])
        l2 = Line(coords=[[49.4362, 111.12, 322.687],[56.0636, 112.369, 318.703]])
        print(p.distance(l))
        print(p.distance(l2))

    def test_foot_point1(self):
        for i in range(5000):
            p = EuclideanPoint([1,2,3])
            l = Line(coords=[[2,3,4],[5,6,7]])
            p.get_foot_point(l)

    def test_foot_point2(self):
        for i in range(5000):
            p = EuclideanPoint([1,2,3])
            l = Line(coords=[[2,3,4],[5,6,7]])
            p.get_foot_point(l)

    def test_foot_point_right(self):
        for i in range(5000):
            p = EuclideanPoint([rand(10),rand(10),rand(10)])
            line = Line(coords=[[rand(10),rand(10),rand(10)],[rand(10),rand(10),rand(10)]])
            ans1 = p.get_foot_point(line)
            ans2 = p.get_foot_point(line)
            for i in range(0,3):
                self.assertTrue(math.fabs(ans1._pos[i] - ans2._pos[i]) < 0.0000001)


if __name__ == "__main__":
    unittest.main()



from pyneval.model.euclidean_point import EuclideanPoint,Line
from pyneval.metric.utils.config_utils import DINF
import time
from anytree import PreOrderIter
from pyneval.model.swc_node import SwcTree

MIN_SIZE = 0.8


def get_nearest_edge(gold_node, test_edge_list, DEBUG=False):
    dis = DINF
    match_line = None
    for test_edge in test_edge_list:
        test_line = Line(swc_node_1=test_edge[0],swc_node_2=test_edge[1])
        eu_node = EuclideanPoint(gold_node._pos)
        tmp_dis = eu_node.distance(test_line)
        tmp_line = test_edge

        if DEBUG:
            eu_node.to_str()
            test_line.to_str()
            print("tmp_dis = {}".format(tmp_dis))

        if tmp_dis < dis:
            dis = tmp_dis
            match_line = tmp_line

    return match_line, dis


def get_test_edge_list(node_list):
    edge_list = []
    for node in node_list:
        if node.is_virtual() or node.parent.is_virtual():
            continue
        edge_list.append(tuple([node, node.parent]))
    return edge_list


#根据边找匹配
def get_match_edges_e(gold_swc_tree=None, test_swc_tree=None, dis_threshold=0.1, DEBUG=False):
    match_edge = set()
    gold_node_list = [node for node in PreOrderIter(gold_swc_tree.root())]
    test_node_list = [node for node in PreOrderIter(test_swc_tree.root())]
    test_edge_list = get_test_edge_list(test_node_list)

    for node in gold_node_list:
        if node.is_virtual() or node.parent.is_virtual():
            continue
        e_node = EuclideanPoint(node._pos)
        e_parent = EuclideanPoint(node.parent._pos)

        line_tuple_a, dis_a = get_nearest_edge(e_node, test_edge_list, DEBUG=False)
        line_tuple_b, dis_b = get_nearest_edge(e_parent, test_edge_list, DEBUG=False)
        if dis_a <= dis_threshold and dis_b <= dis_threshold:
            match_edge.add(tuple([node, node.parent]))
            if DEBUG:
                with open('normal.txt', 'a') as f:
                    f.write("\nnode = {} {}\nnode.parent = {} {}\nnode_line = {},{}\nnode_p_line = {},{}\n".format(
                        node.get_id(), node._pos,
                        node.parent.get_id(),node.parent._pos,
                        line_tuple_a[0]._pos, line_tuple_a[1]._pos,
                        line_tuple_b[0]._pos, line_tuple_b[1]._pos,
                    ))
    return match_edge


def length_metric_2_2(gold_swc_tree=None, test_swc_tree=None, dis_threshold=0.1, DEBUG=False):
    match_edges = get_match_edges_e(gold_swc_tree, test_swc_tree,dis_threshold, DEBUG=True)

    match_length = 0.0
    for line_tuple in match_edges:
        match_length += line_tuple[0].parent_distance()

    gold_total_length = round(gold_swc_tree.length(),8)
    match_length = round(match_length,8)

    if DEBUG:
        print("match_length b = {}, gold_total_length = {}"
              .format(match_length, gold_total_length))
    return match_length/gold_total_length

if __name__ == "__main__":
    goldtree = SwcTree()
    goldtree.load("D:\gitProject\mine\PyNeval\\test\data_example\gold\\30_18_10_gold.swc")

    testTree = SwcTree()
    testTree.load("D:\gitProject\mine\PyNeval\\test\data_example\\test\\30_18_10_test.swc")
    start = time.time()
    print(length_metric_2_2(gold_swc_tree=goldtree,
                        test_swc_tree=testTree))

    print(time.time() - start)


# @Author: guanwanxian
# @Date:   1970-01-01T08:00:00+08:00
# @Email:  guanwanxian@zju.edu.cn
# @Last modified by:   guanwanxian
# @Last modified time: 2016-12-27T20:24:14+08:00



# import the necessary packages
# import imutils

def pyramid(image, scale=1.5, minSize=(30, 30)):
	original_proportion = 1

	# yield the original image
	yield (original_proportion, image)

	# keep looping over the pyramid
	while True:
		# compute the new dimensions of the image and resize it
		w = int(image.shape[1] / scale)
		image = imutils.resize(image, width=w)
		original_proportion = original_proportion * scale
		# if the resized image does not meet the supplied minimum
		# size, then stop constructing the pyramid
		if image.shape[0] < minSize[1] or image.shape[1] < minSize[0]:
			break

		# yield the next image in the pyramid
		yield (original_proportion, image)

def sliding_window(image, stepSize, windowSize):
	# slide a window across the image
	for y in xrange(0, image.shape[0], stepSize):
		for x in xrange(0, image.shape[1], stepSize):
			# yield the current window
			yield (x, y, image[y:y + windowSize[1], x:x + windowSize[0]])

def sliding_window_2d(image, step_size_h, step_size_w, windowSize):
	# slide a window across the image
	for y in range(0, image.shape[0], step_size_h):
		for x in range(0, image.shape[1], step_size_w):
			# yield the current window
			yield (x, y, image[y:y + windowSize[1], x:x + windowSize[0]])


def slidingWindow3D(image, step_size, window_size):
	''' slide a window across the image
		Input:
			image (3d numpy array): z,y,x order.
			step_size (int).
			window_size (int).
		Output:
			(3d numpy array).
	'''
	for y in range(0, image.shape[1], step_size):
		for x in range(0, image.shape[2], step_size):
			for z in range(0, image.shape[0], step_size):
				img = image[z:z+window_size, y:y+window_size, x:x+window_size]
				if img.shape[0]==window_size and img.shape[1]==window_size and img.shape[2]==window_size:
					yield (z,y,x,img)



#!/usr/bin/python
# -*- coding:utf-8 -*-

"""
* @Filename: mail.py
* @Author: Zhefeng Wang(jevons@gmail.com)
* @Created on: 2016年 04月 24日 星期日 21:37:54 CST
* @desc: function to send emails
* @history:
"""

import sys,os
from imp import reload
reload(sys)

from email import encoders
from email.header import Header
from email.mime.text import MIMEText
from email.utils import parseaddr, formataddr

import smtplib

def notifyMe(subject, mailto='425172133@qq.com',content=u'官万先\r\nguanwanxian@zju.edu.cn'):

    from_addr = "guanwanxian@zju.edu.cn"
    password = "64544861"

    #to_addr = "972562104@qq.com"
    to_addr = mailto
    smtp_server = "smtp.zju.edu.cn"

    msg = MIMEText(content, 'plain', 'utf-8')
    msg['From'] = '官万先<%s>' % from_addr
    msg['To'] = mailto
    msg['Subject'] = Header(subject, 'utf-8').encode()

    server = smtplib.SMTP(smtp_server, 25)
    server.set_debuglevel(1)
    server.login(from_addr, password)
    server.sendmail(from_addr, [to_addr], msg.as_string())
    server.quit()

if __name__ == "__main__":
    mailto = "972562104@qq.com"
    subject = "你好"
    content = "Hello World!"
    send_email(mailto, subject, content)



import numpy as np
import math
from rtree import index

from pyneval.model import euclidean_point as euc_p
from pyneval.metric.utils import config_utils
from pyneval.model import swc_node


def get_match_edges(gold_swc_tree=None, test_swc_tree=None,
                    radius_threshold=-1.0, length_threshold=0.2,
                    debug=False):
    """
    get matched edge set
    Args:
        gold_swc_tree(SwcTree):
        test_swc_tree(SwcTree):
        radius_threshold(float): threshold of key point radius
        length_threshold(float): threshold of length of the matching edges
        debug(bool): list debug info ot not
    Returns:
        match_edge(set): include all edge that are matched in gold swc tree
            edge(tuple): every edge is a tuple contains two side nodes of a edge
        test_match_length(float): it's the total length of matched area on test swc tree
    Raises:
        [Error: ] Max id in test swc tree too large
            the size of vis list is depend on the maximum id of the tree.
            this number couldn't be too large

    """

    match_edge = set()
    edge_use_dict = {}
    id_rootdis_dict = {}
    test_match_length = 0.0

    test_rtree = get_edge_rtree(test_swc_tree)
    id_edge_dict = get_idedge_dict(test_swc_tree)
    gold_node_list = gold_swc_tree.get_node_list()
    test_node_list = test_swc_tree.get_node_list()

    # node num need to be larger than the max id
    test_maxum = 0
    for node in test_node_list:
        id_rootdis_dict[node.get_id()] = node.root_length
        test_maxum = max(test_maxum, node.get_id())

    try:
        vis_list = np.zeros(test_maxum+5, dtype='int8')
        test_swc_tree.get_lca_preprocess(node_num=test_maxum+5)
    except:
        raise Exception("[Error: ] Max id in test swc tree too large")

    for node in gold_node_list:
        if node.is_virtual() or node.parent.is_virtual():
            continue

        rad_threshold1, rad_threshold2 = cal_rad_threshold(radius_threshold, node.radius(), node.parent.radius())

        line_tuple_a_set = get_nearby_edges(rtree=test_rtree, point=node, id_edge_dict=id_edge_dict,
                                            threshold=rad_threshold1, not_self=False, debug=debug)
        line_tuple_b_set = get_nearby_edges(rtree=test_rtree, point=node.parent, id_edge_dict=id_edge_dict,
                                            threshold=rad_threshold2, not_self=False, debug=debug)

        done = False
        for line_tuple_a_dis in line_tuple_a_set:
            if done:
                break
            for line_tuple_b_dis in line_tuple_b_set:
                line_tuple_a = line_tuple_a_dis[0]
                dis_a = line_tuple_a_dis[1]
                line_tuple_b = line_tuple_b_dis[0]
                dis_b = line_tuple_b_dis[1]

                test_length = get_lca_length(test_swc_tree, \
                               line_tuple_a, \
                               line_tuple_b, \
                               euc_p.Line(e_node_1=node.get_center(),
                                          e_node_2=node.parent.get_center()),
                               id_rootdis_dict)
                gold_length = node.parent_distance()

                if test_length == config_utils.DINF:
                    continue

                len_threshold1 = cal_len_threshold(length_threshold, gold_length)
                if not (dis_a <= rad_threshold1 and dis_b <= rad_threshold2):
                    if debug:
                        print(node.get_id(), dis_a, rad_threshold1, dis_b, rad_threshold2, "error1")
                    continue
                if not (math.fabs(test_length - gold_length) < len_threshold1):
                    if debug:
                        print(node.get_id(), "error2")
                    continue
                if not is_route_clean(gold_swc_tree=test_swc_tree,
                                      gold_line_tuple_a=line_tuple_a, gold_line_tuple_b=line_tuple_b,
                                      node1=node, node2=node.parent,
                                      edge_use_dict=edge_use_dict, vis_list= vis_list, debug=debug):
                    if debug:
                        print(node.get_id(), "error3")
                    continue
                match_edge.add(tuple([node, node.parent]))
                test_match_length += test_length
                done = True
                break

        if not done:
            node._type = 9
            if debug:
                print("{} not done".format(node.get_id()))

    return match_edge, test_match_length


def get_edge_rtree(swc_tree=None):
    """
    build a rtree based on the swc tree
    Args：
        swc_tree(Swc Tree): Swc_Tree, to build rtree
    Return:
        rtree(rtree index)
    """
    swc_tree_list = swc_tree.get_node_list()
    p = index.Property()
    p.dimension = 3
    rtree = index.Index(properties=p)
    for node in swc_tree_list:
        if node.is_virtual() or node.parent.is_virtual():
            continue

        rtree.insert(node.get_id(), get_bounds(node, node.parent, extra=node.radius()))
    return rtree


def get_bounds(point_a, point_b, extra=0):
    """
    get bounding box of a segment
    Args:
        point_a: two points to identify the square
        point_b:
        extra: float, a threshold
    Return:
        res(tuple):
    """
    point_a = np.array(point_a.get_center()._pos)
    point_b = np.array(point_b.get_center()._pos)
    res = (np.where(point_a > point_b, point_b, point_a) - extra).tolist() + (np.where(point_a > point_b, point_a, point_b) + extra).tolist()

    return tuple(res)


def get_idedge_dict(swc_tree=None):
    '''
    get a dict, mapping from id to edge.
    id_edge_dict[swc_node1.id] = tuple(swc_node1, swc_node2)
    Args:
        swc_tree:
    Return:
        id_edge_dict(Dict)
        key: son node's id
        value: tuple(node, node.parent)
    '''
    id_edge_dict = {}
    swc_tree_list = swc_tree.get_node_list()
    for node in swc_tree_list:
        if node.is_virtual() or node.parent.is_virtual():
            continue
        id_edge_dict[node.get_id()] = tuple([node, node.parent])
    return id_edge_dict


def cal_rad_threshold(rad_threshold, dis1, dis2):
    """
    TODO: Negative value of threshold is not recommand. Waiting to be modified.
    """
    if rad_threshold < 0:
        return -rad_threshold*dis1, -rad_threshold*dis2
    else:
        return rad_threshold, rad_threshold


def cal_len_threshold(len_threshold, length):
    return length * len_threshold


def get_nearby_edges(rtree, point, id_edge_dict, threshold, not_self=False, debug=False):
    '''
    find the close enough edges of a node base on rtree
    sorted by distance
    Args:
        rtree(): an rtree describe the target edge set
        point: the point to get nearby edges
        id_edge_dict:map between id and line tuple(edge)
        threshold: the ceiling of the distance between point and edge
        not_self: exclude self, used in overlap detect
        debug:
    Returns:
         a list of tuple(edge, dis). Sorted according to distance to the point
    level: 1
    '''
    point_box = (point.get_x() - threshold, point.get_y() - threshold, point.get_z() - threshold,
                 point.get_x() + threshold, point.get_y() + threshold, point.get_z() + threshold)
    hits = list(rtree.intersection(point_box))
    nearby_edges = []

    for h in hits:
        line_tuple = id_edge_dict[h]
        if debug:
            print("\npoint = id{} poi{}, line_a = id{} poi{}, line_b = id{} poi{}".format(
                point.get_id(), point.get_center()._pos,
                line_tuple[0].get_id(), line_tuple[0].get_center()._pos,
                line_tuple[1].get_id(), line_tuple[1].get_center()._pos)
            )
        e_point = point.get_center()

        # if two sides of line_tuple is in the same position, ignore this line
        if line_tuple[0].get_center().distance(line_tuple[1].get_center()) == 0:
            continue

        new_d = e_point.distance(euc_p.Line(e_node_1=line_tuple[0].get_center(), e_node_2=line_tuple[1].get_center()))
        if not_self and new_d == 0:
            continue
        nearby_edges.append(tuple([line_tuple, new_d]))
    nearby_edges.sort(key=lambda x: x[1])
    return nearby_edges


def get_lca_length(gold_swc_tree, gold_line_tuple_a, gold_line_tuple_b, test_line, id_rootdis_dict):
    """
    calculate the length of the path on gold_swc_tree corresponding to test line
    This function get the foot of two side nodes of test_line first.
    than calculate the distance between two foots.
    Args:
        gold_swc_tree(SwcTree): swc tree that are calculated.
        gold_line_tuple_a(tuple): a list of two nodes describe a edge
        gold_line_tuple_b(tuple): a list of two nodes describe a edge
        test_line(Line): Line object defined in pyneval/model/euclidean_point
        id_rootdis_dict(Dict): a dict [int to float], get nodes' distance to root by its id
    Return:
        lca_length(float): distance of two foot of side nodes of test line
    """
    point_a, point_b = test_line.get_points()

    gold_line_a = euc_p.Line(e_node_1=gold_line_tuple_a[0].get_center(),
                             e_node_2=gold_line_tuple_a[1].get_center())
    gold_line_b = euc_p.Line(e_node_1=gold_line_tuple_b[0].get_center(),
                             e_node_2=gold_line_tuple_b[1].get_center())

    foot_a = point_a.get_closest_point(gold_line_a)
    foot_b = point_b.get_closest_point(gold_line_b)

    # if two foots lay on the same edge, pass
    if gold_line_tuple_a[0].get_id() == gold_line_tuple_b[0].get_id() and \
        gold_line_tuple_a[1].get_id() == gold_line_tuple_b[1].get_id():
        return foot_a.distance(foot_b)

    lca_id = gold_swc_tree.get_lca(gold_line_tuple_a[0].get_id(), gold_line_tuple_b[0].get_id())
    if lca_id is None or lca_id < 1:
        return config_utils.DINF
    try:
        lca_length2 = id_rootdis_dict[gold_line_tuple_a[0].get_id()] + \
                      id_rootdis_dict[gold_line_tuple_b[0].get_id()] - \
                      id_rootdis_dict[lca_id]*2
    except:
        print(id_rootdis_dict)
        raise Exception("[error: lca_id={}]".format(
            lca_id
        ))
    # get nodes on the route, make sure no extra node
    route_list_a = get_route_node(gold_line_tuple_a[0], lca_id)
    route_list_b = get_route_node(gold_line_tuple_b[0], lca_id)

    lca_length = 0.0
    # if (gold_line_tuple_a[1] in route_list_a) != (gold_line_tuple_a[0].get_id() != lca_id):
    #     print(gold_line_tuple_a[1] in route_list_a, gold_line_tuple_a[0].get_id() != lca_id)
    if gold_line_tuple_a[0].get_id() != lca_id:
        route_list_a.remove(gold_line_tuple_a[0])
        lca_length2 -= gold_line_tuple_a[0].parent_distance()
        lca_length += foot_a.distance(gold_line_tuple_a[1].get_center())
        lca_length2 += foot_a.distance(gold_line_tuple_a[1].get_center())
    else:
        lca_length += foot_a.distance(gold_line_tuple_a[0].get_center())
        lca_length2 += foot_a.distance(gold_line_tuple_a[0].get_center())
    # if (gold_line_tuple_b[1] in route_list_b) != (gold_line_tuple_b[0].get_id() != lca_id):
    #     print(gold_line_tuple_b[1] in route_list_b, gold_line_tuple_b[0].get_id() != lca_id)
    if gold_line_tuple_b[0].get_id() != lca_id:
        route_list_b.remove(gold_line_tuple_b[0])
        lca_length2 -= gold_line_tuple_b[0].parent_distance()
        lca_length += foot_b.distance(gold_line_tuple_b[1].get_center())
        lca_length2 += foot_b.distance(gold_line_tuple_b[1].get_center())
    else:
        lca_length += foot_b.distance(gold_line_tuple_b[0].get_center())
        lca_length2 += foot_b.distance(gold_line_tuple_b[0].get_center())

    route_list = route_list_a + route_list_b
    for node in route_list:
        if node.get_id() == lca_id:
            continue
        lca_length += node.parent_distance()

    return lca_length


def is_route_clean(gold_swc_tree, gold_line_tuple_a, gold_line_tuple_b, node1, node2, edge_use_dict, vis_list, debug):
    """
    Check if all edge between two euc nodes are totally unused.
    for any edge, we constructed a data structure call interval, recording witch parts of the edge are used.
    so this function will call add_interval, is_intered to change the useage of the edge.
    Args:
        gold_swc_tree(SwcTree) swc tree that are calculated.
        gold_line_tuple_a(tuple): a list of two nodes describe a edge
        gold_line_tuple_b(tuple): a list of two nodes describe a edge
        node1(Swc_Node): one node side
        node2(Swc_Node): the other node side
        edge_use_dict(Dict):
            example: [swc_node, list[interval_1, interval_2]]
            interval is a tuple of two integers,
            edge_use_dict shows which part of edge between swc_node and swc_node.parent has been used
        vis_list: list, check if whole edge has been used
    Return:
        True/False
    """
    point_a = node1.get_center()
    point_b = node2.get_center()
    gold_line_a = euc_p.Line(e_node_1=gold_line_tuple_a[0].get_center(),
                             e_node_2=gold_line_tuple_a[1].get_center())
    gold_line_b = euc_p.Line(e_node_1=gold_line_tuple_b[0].get_center(),
                             e_node_2=gold_line_tuple_b[1].get_center())

    foot_a = point_a.get_closest_point(gold_line_a)
    foot_b = point_b.get_closest_point(gold_line_b)

    # if two foots lay on the same edge, pass
    if gold_line_tuple_a[0].get_id() == gold_line_tuple_b[0].get_id() and \
        gold_line_tuple_a[1].get_id() == gold_line_tuple_b[1].get_id():
            total_length = gold_line_tuple_a[0].distance(gold_line_tuple_a[1])
            start = foot_a.distance(gold_line_tuple_a[0].get_center()) / total_length
            end = foot_b.distance(gold_line_tuple_a[0].get_center()) / total_length
            if debug:
                print("node = {} {}\nnode.parent = {} {}\nnode_line = {},{} usage = {} {}\n".format(
                        node1.get_id(), node1.get_center()._pos,
                        node2.get_id(), node2.get_center()._pos,
                        gold_line_tuple_a[0].get_id(), gold_line_tuple_a[1].get_id(), start, end,
                    ))
            if start > end:
                start, end = end, start
            end -= config_utils.FLOAT_ERROR
            if vis_list[gold_line_tuple_a[0].get_id()] == 1:
                return False
            if add_interval(edge_use_dict, gold_line_tuple_a[0], tuple([start, end])):
                if start < end:
                    edge_use_dict[gold_line_tuple_a[0]].add(tuple([start, end]))
                return True
            return False

    # not on the same edge, get lca first
    lca_id = gold_swc_tree.get_lca(gold_line_tuple_a[0].get_id(), gold_line_tuple_b[0].get_id())
    if lca_id is None:
        raise Exception("[Error:] No Lca found")

    # get nodes on the route, make sure no extra node
    route_list_a = get_route_node(gold_line_tuple_a[0], lca_id)
    route_list_b = get_route_node(gold_line_tuple_b[0], lca_id)

    if gold_line_tuple_a[1] in route_list_a:
        route_list_a.remove(gold_line_tuple_a[0])
        start_a = foot_a.distance(gold_line_tuple_a[0].get_center()) / gold_line_tuple_a[0].parent_distance()
        end_a = 1.0
    else:
        start_a = 0.0
        end_a = foot_a.distance(gold_line_tuple_a[0].get_center()) / gold_line_tuple_a[0].parent_distance()

    if gold_line_tuple_b[1] in route_list_b:
        route_list_b.remove(gold_line_tuple_b[0])
        start_b = foot_b.distance(gold_line_tuple_b[0].get_center()) / gold_line_tuple_b[0].parent_distance()
        end_b = 1.0
    else:
        start_b = 0.0
        end_b = foot_b.distance(gold_line_tuple_b[0].get_center()) / gold_line_tuple_b[0].parent_distance()

    if debug:
        print("\nnode = {} {}\nnode.parent = {} {}\nnode_line = {},{} usage = {} {}\nnode_p_line = {},{} usage = {} {}\n".format(
            node1.get_id(), node1._pos,
            node2.get_id(), node2._pos,
            gold_line_tuple_a[0].get_id(), gold_line_tuple_a[1].get_id(), start_a, end_a,
            gold_line_tuple_b[0].get_id(), gold_line_tuple_b[1].get_id(), start_b, end_b,
        ))

    # for each internal left point is included, right is not
    end_a -= config_utils.FLOAT_ERROR
    end_b -= config_utils.FLOAT_ERROR
    route_list = route_list_a + route_list_b
    for node in route_list:
        if node.get_id() == lca_id:
            continue
        if vis_list[node.get_id()] == 1 or exist(edge_use_dict, node):
            return False

    if add_interval(edge_use_dict, gold_line_tuple_a[0], tuple([start_a, end_a])) and \
       add_interval(edge_use_dict, gold_line_tuple_b[0], tuple([start_b, end_b])) and \
        vis_list[gold_line_tuple_a[0].get_id()] == 0 and \
        vis_list[gold_line_tuple_b[0].get_id()] == 0:
        if start_a < end_a:
            edge_use_dict[gold_line_tuple_a[0]].add(tuple([start_a, end_a]))
        if start_b < end_b:
            edge_use_dict[gold_line_tuple_b[0]].add(tuple([start_b, end_b]))

        for node in route_list:
            if node.get_id() == lca_id:
                continue
            vis_list[node.get_id()] = 1
        return True
    return False


def is_intered(inter1, inter2):
    """
    check if two inters has intersection
    inter are just tuplas for two side nodes: both left and right are closed interval [... , ...]
    """
    if inter1[0] == inter1[1] or inter2[0] == inter2[1]:
        return False
    if inter1[0] <= inter2[0] <= inter1[1] or inter1[0] <= inter2[1] <= inter1[1] or \
       inter2[0] <= inter1[0] <= inter2[1] or inter2[0] <= inter1[1] <= inter2[1]:
        return True
    return False


def add_interval(dic, edge, interval):
    """
        add a interval to the edge.
        demonstrate this part of the edge has been used.

    """
    if edge not in dic.keys():
        dic[edge] = set()
    if interval[0] > interval[1]:
        return True
    for inter in dic[edge]:
        if is_intered(inter, interval):
            return False
    return True


def get_route_node(current_node, lca_id):
    """
    get all node from current node to the LCA node
    """
    res_list = []
    while not current_node.is_virtual() and not current_node.get_id() == lca_id:
        res_list.append(current_node)
        current_node = current_node.parent

    res_list.append(current_node)
    return res_list


def exist(dic, edge):
    """
    check if any part of edge is used
    """
    if edge not in dic.keys():
        return False
    if len(dic[edge]) > 0:
        return True
    return False



import numpy as np

import os
import urllib
import gzip
import pickle

def unpickle(file):
    fo = open(file, 'rb')
    dict = pickle.load(fo, encoding='latin1')
    fo.close()
    return dict['data'], dict['labels']

def cifar_generator(filenames, batch_size, data_dir):
    all_data = []
    all_labels = []
    for filename in filenames:        
        data, labels = unpickle(data_dir + '/' + filename)
        all_data.append(data)
        all_labels.append(labels)

    images = np.concatenate(all_data, axis=0)
    labels = np.concatenate(all_labels, axis=0)

    def get_epoch():
        rng_state = np.random.get_state()
        np.random.shuffle(images)
        np.random.set_state(rng_state)
        np.random.shuffle(labels)

        for i in range(int(len(images) / batch_size)):
            yield (images[i*batch_size:(i+1)*batch_size], labels[i*batch_size:(i+1)*batch_size])

    return get_epoch


def load(batch_size, data_dir):
    return (
        cifar_generator(['data_batch_1','data_batch_2','data_batch_3','data_batch_4','data_batch_5'], batch_size, data_dir), 
        cifar_generator(['test_batch'], batch_size, data_dir)
    )



import importlib
import argparse
import Utils

class Parser:
    def __init__(self):
        ## Parse command line parameters
        parser = argparse.ArgumentParser(description='iwgan framework.')
        subparsers = parser.add_subparsers(help='subcomand description.')

        parser_train = subparsers.add_parser('train', help='start training a mode..')
        parser_continue = subparsers.add_parser('continue', help='continue training a model.')
        parser_test = subparsers.add_parser('test', help='test a model.')

        parser_train.add_argument(
                '-m',
                metavar='model_directory_name',
                nargs=1,
                required=True,
                help='model directory name located under "model" directory')
        parser_train.add_argument(
        		'-e',
        		metavar='extra_comment',
        		nargs=1,
                help='extra comment about this experiment')
        parser_train.set_defaults(func=self.trainParser)


        parser_continue.add_argument(
                '-m',
                metavar='model_directory_name',
                nargs=1,
                required=True,
                help='model directory name located under "model" directory')
        parser_continue.add_argument(
                '-d',
                metavar='experiment_dir',
                nargs=1,
                required=True,
                help='experiment directory name located under "output" directory')
        parser_continue.add_argument(
        		'-s',
        		metavar='step',
        		type=int,
        		nargs=1,
                help='continue from specified step, if not given use the latest checkpoint')
        parser_continue.add_argument(
                '-n',
                action='store_true',
                help='whether write tensorboard log to new directory, if not specified: false')
        parser_continue.add_argument(
        		'-e',
        		metavar='extra_comment',
        		nargs=1,
                help='extra comment about this experiment, only work when -n is set true')


        parser_continue.set_defaults(func=self.finetuneParser)

        parser_test.add_argument(
                '-m',
                metavar='model_directory_name',
                nargs=1,
                required=True,
                help='model directory name located under "model" directory')
        parser_test.add_argument(
                '-d',
                metavar='experiment_dir',
                nargs=1,
                required=True,
                help='experiment directory name located under "output" directory')
        parser_test.add_argument(
                '-s',
                metavar='step',
                nargs=1,
                help='continue from specified step, if not given use the latest checkpoint')
        parser_test.add_argument(
                '-p',
                metavar='self.env.test_type',
                nargs=1,
                required=True,
                help='1.generate fake images.')
        parser_test.set_defaults(func=self.testParser)

        self.parser = parser
        # 用于存放配置信息
        self.env = None

    def parseArgs(self):
        self.args = self.parser.parse_args()
        self.args.func(self.args)
        return self.env

    def commonParser(self, args):
        ## load model package
        model_dir_name = args.m[0]
        model_path = 'model.{}.model'.format(model_dir_name)
        model_factory = importlib.import_module(model_path)

        ## load config file
        config_file_path = 'model/{}/config.json'.format(model_dir_name)
        config_data = Utils.json2dict(config_file_path)
        self.env = model_factory.Environment(config_data)

        ## create root directory
        Utils.setDir(self.env.root_dir)

        ## 设置其他环境变量
        self.env.config_file_path = config_file_path
        self.env.model_dir_name = model_dir_name
        self.env.model_factory = model_factory

    def trainParser(self, args):
        self.commonParser(args)

        self.env.do_train = True
        self.env.continue_from_pre = False
        extra_comment = None
        if args.e:
            extra_comment = args.e[0]

        exp_dir = Utils.generate_expid(self.env.model_dir_name, extra_comment)
        self.env.exp_dir_path = '/'.join([self.env.root_dir,exp_dir])
        Utils.resetDir(self.env.exp_dir_path)

        self.env.log_dir_path = '/'.join([self.env.exp_dir_path,self.env.log_dir])
        Utils.setDir(self.env.log_dir_path)

        self.env.checkpoint_dir_path= '/'.join([self.env.exp_dir_path,self.env.checkpoint_dir])
        Utils.setDir(self.env.checkpoint_dir_path)

        self.env.specified_checkpoint_dir_path= '/'.join([self.env.exp_dir_path,self.env.specified_checkpoint_dir])
        Utils.setDir(self.env.specified_checkpoint_dir_path)

    def finetuneParser(self, args):
        self.commonParser(args)

        self.env.do_train = True
        self.env.continue_from_pre = True

        if args.n: # generate
            old_exp_dir = args.d[0]

            extra_comment = None
            if args.e:
                extra_comment = args.e[0]
            new_exp_dir = Utils.generate_expid(self.env.model_dir_name, extra_comment)
            old_exp_path = '/'.join([self.env.root_dir,old_exp_dir])
            new_exp_path = '/'.join([self.env.root_dir,new_exp_dir])
            Utils.setDir(new_exp_path)
            os.system('cp -r %s/*  %s'%(old_exp_path,new_exp_path))
            self.env.exp_dir_path = new_exp_path
            Utils.testDir(self.env.exp_dir_path)
        else:
            exp_dir = args.d[0]
            self.env.exp_dir_path = '/'.join([self.env.root_dir,exp_dir])
            Utils.testDir(self.env.exp_dir_path)


        self.env.log_dir_path = '/'.join([self.env.exp_dir_path,self.env.log_dir])
        Utils.testDir(self.env.log_dir_path)

        self.env.checkpoint_dir_path= '/'.join([self.env.exp_dir_path,self.env.checkpoint_dir])
        Utils.testDir(self.env.checkpoint_dir_path)

        self.env.specified_checkpoint_dir_path= '/'.join([self.env.exp_dir_path,self.env.specified_checkpoint_dir])
        Utils.testDir(self.env.specified_checkpoint_dir_path)

        self.env.restore_step=None
        if args.s:
            self.env.restore_step = args.s[0]

    def testParser(self, args):
        self.commonParser(args)

        self.env.do_train = False
        self.env.continue_from_pre = True

        exp_dir = args.d[0]
        self.env.exp_dir_path = '/'.join([self.env.root_dir,exp_dir])
        Utils.testDir(self.env.exp_dir_path)


        self.env.log_dir_path = '/'.join([self.env.exp_dir_path,self.env.log_dir])
        Utils.testDir(self.env.log_dir_path)

        self.env.checkpoint_dir_path= '/'.join([self.env.exp_dir_path,self.env.checkpoint_dir])
        Utils.testDir(self.env.checkpoint_dir_path)

        self.env.specified_checkpoint_dir_path= '/'.join([self.env.exp_dir_path,self.env.specified_checkpoint_dir])
        Utils.testDir(self.env.specified_checkpoint_dir_path)

        self.env.restore_step=None
        if args.s:
            self.env.restore_step = args.s[0]
        self.env.test_type = args.p[0]



import os

from pyneval.errors.exceptions import InvalidTiffFileError
from pyneval.metric.utils.klib.TiffFile import TiffFile, imread, imsave


def is_tiff_file(tiff_file):
    return tiff_file[-4:] in (".tif", ".TIF")


def read_tiff(tiff_path):
    """
    Args:
        tiff_path: a tiff file path

    Returns:
        tiff image object
    """
    if not os.path.isfile(tiff_path) or not is_tiff_file(tiff_path):
        raise InvalidTiffFileError(tiff_path)
    return imread(tiff_path)


def read_tiffs(tiff_paths):
    """
    Args:
        tiff_paths: tiff_path file path for directs that contain tiff files
    Returns:
        tiff image objects
    """
    # load all tiff file paths
    tiff_files = []
    if os.path.isfile(tiff_paths):
        if not is_tiff_file(tiff_files):
            print (tiff_paths + "is not a tif file")
            return None
        tiff_files.append(tiff_paths)
    else:
        for root, _, files in os.walk(tiff_paths):
            for file in files:
                f = os.path.join(root, file)
                if is_tiff_file(f):
                    tiff_files.append(f)
    # load image objects
    tiff_list = []
    for tiff_file in tiff_files:
        tiff_list.append(imread(tiff_file))
    return tiff_list



# -*- coding: utf-8 -*-
"""a geometry metric method, SSD metric

This module implement a geometry metric method called Substantial Spatial Distance.
relative paper: https://doi.org/10.1038/nbt.1612
title, auther

Example:
    In command line:
    $ pyneval --gold ./data\\test_data\\ssd_data\\gold\\c.swc
              --test .\\data\\test_data\\ssd_data\\test\\c.swc
              --metric ssd_metric
Attributes:
    None

Todos:
    None
"""
import sys

import jsonschema

from pyneval.model import swc_node
from pyneval.tools import re_sample
from pyneval.metric.utils import point_match_utils
from pyneval.metric.utils import basic_utils
from pyneval.metric.utils.metric_manager import get_metric_manager

metric_manager = get_metric_manager()


class SsdMetric(object):
    """
    ssd metric
    """

    def __init__(self, config):
        self.debug = config["debug"] if config.get("debug") is not None else False
        self.threshold_mode = config["threshold_mode"]
        self.ssd_threshold = config["ssd_threshold"]
        self.up_sample_threshold = config["up_sample_threshold"]
        self.scale = config["scale"]

    def get_mse(self, src_tree, tar_tree, ssd_threshold=2.0, mode=1):
        """ calculate the minimum square error of two trees
        find the closest node on the tar_tree for each node on the src tree, calculate the average
        distance of these node pairs.

        Args:
            src_tree(SwcTree):
            tar_tree(SwcTree):
            ssd_threshold(float): distance will be count into the res
                only if two nodes' distance is larger than this threshold.
            mode(1 or 2):
                1 means static threshold, equal to ssd_threshold.
                2 means dynamic threshold, equal to ssd_threshold * src_node.threshold

        Returns:
            dis(float): average minimum distance of node, distance must be larger than ssd_threshold
            num(float): The number of pairs of nodes that are counted when calculating distance

        Raise:
            None
        """
        dis, num = 0, 0
        kdtree, pos_node_dict = point_match_utils.create_kdtree(tar_tree.get_node_list())
        for node in src_tree.get_node_list():
            if node.is_virtual():
                continue
            target_pos = kdtree.search_knn(list(node.get_center_as_tuple()), k=1)[0]
            target_node = pos_node_dict[tuple(target_pos[0].data)]

            cur_dis = target_node.distance(node)

            if mode == 1:
                threshold = ssd_threshold
            else:
                threshold = ssd_threshold * node.radius()

            if cur_dis >= threshold:
                node._type = 9
                dis += cur_dis
                num += 1
        try:
            dis /= num
        except ZeroDivisionError:
            dis = num = 0
        return dis, num

    def run(self, gold_swc_tree, test_swc_tree):
        gold_swc_tree.rescale(self.scale)
        test_swc_tree.rescale(self.scale)
        u_gold_swc_tree = re_sample.up_sample_swc_tree(swc_tree=gold_swc_tree,
                                                       length_threshold=self.up_sample_threshold)
        u_test_swc_tree = re_sample.up_sample_swc_tree(swc_tree=test_swc_tree,
                                                       length_threshold=self.up_sample_threshold)
        u_gold_swc_tree.set_node_type_by_topo(root_id=1)
        u_test_swc_tree.set_node_type_by_topo(root_id=5)

        g2t_score, g2t_num = self.get_mse(src_tree=u_gold_swc_tree, tar_tree=u_test_swc_tree,
                                     ssd_threshold=self.ssd_threshold, mode=self.threshold_mode)
        t2g_score, t2g_num = self.get_mse(src_tree=u_test_swc_tree, tar_tree=u_gold_swc_tree,
                                     ssd_threshold=self.ssd_threshold, mode=self.threshold_mode)

        if self.debug:
            print("recall_num = {}, pre_num = {}, gold_tot_num = {}, test_tot_num = {} {} {}".format(
                g2t_num, t2g_num, u_gold_swc_tree.size(), u_test_swc_tree.size(), gold_swc_tree.length(),
                test_swc_tree.length()
            ))

        res = {
            "ssd_score": (g2t_score + t2g_score) / 2,
            "recall": 1 - g2t_num / u_gold_swc_tree.size(),
            "precision": 1 - t2g_num / u_test_swc_tree.size(),
            "f1_score": basic_utils.get_f1score(
                recall=1 - g2t_num / u_gold_swc_tree.size(),
                precision=1 - t2g_num / u_test_swc_tree.size()
            )
        }

        return res, u_gold_swc_tree, u_test_swc_tree


@metric_manager.register(
    name="ssd",
    config="ssd_metric.json",
    desc="minimum square error between resampled gold and test trees",
    alias=['SM'],
    public=True,
)
def ssd_metric(gold_swc_tree: swc_node.SwcTree, test_swc_tree: swc_node.SwcTree, config: dict):
    """Main function of SSD metric.
    Args:
        gold_swc_tree(SwcTree):
        test_swc_tree(SwcTree):
        config(Dict):
            The keys of 'config' is the name of configs, and the items are config values
    Example:
        test_tree = swc_node.SwcTree()
        gold_tree = swc_node.SwcTree()
        gold_tree.load("..\\..\\data\\test_data\\ssd_data\\gold\\c.swc")
        test_tree.load("..\\..\\data\\test_data\\ssd_data\\test\\c.swc")
        score, recall, precision = ssd_metric(gold_swc_tree=gold_tree,
                                              test_swc_tree=test_tree,
                                              config=config)
    Returns:
        tuple: contain three values to demonstrate metric result
            avg_score(float): average distance of nodes between gold and test swc trees.
            precision(float): percentage of nodes that are matched compared to test tree
            recall(float): percentage of nodes that are matched compared to gold tree

    Raises:
        None
    """
    ssd = SsdMetric(config)
    return ssd.run(gold_swc_tree, test_swc_tree)




import os
from functools import reduce
from collections import  deque
import numpy as np
import scipy as sp
from numpy import linalg as LA
from scipy.spatial import distance_matrix

from Transformations import rotation_matrix, superimposition_matrix
from SWCExtractor import Vertex
from Obj3D import Point3D, Sphere, Cone, calculateBound, calScaleRatio
from Utils import  Timer
import Draw3DTools
import ImageUtils

def getRandChildNumber():
    ''' Random generate children number of a tree node
        Input:
            None
        Output:
            (int) : Children number
    '''
    return np.random.choice([1,2,3,4], p=[0.5, 0.35, 0.1, 0.05])

def getChildRadius(depth, max_depth):
    if depth==0: # root
        return np.random.choice([3,4,5], p=[0.25,0.5,0.25])
    else:
        return np.random.choice([1,2,3,4,5], p=[0.05, 0.2, 0.35, 0.35, 0.05])

def getChildLength(depth, max_depth):
    ''' 子节点距离父节点的长度
    '''
    return 25 + (max_depth-depth) + np.random.randint(0,11)

def getNodeFromMark(mark, pos, MIN_DISTANCE, MAX_DISTANCE, mark_shape, use_parent=False, parent_pos=None):

    # Calculate general search range
    x,y,z = pos
    bbox = [x-MAX_DISTANCE, y-MAX_DISTANCE, z-MAX_DISTANCE, x+MAX_DISTANCE+1, y+MAX_DISTANCE+1, z+MAX_DISTANCE+1] # xmin,ymin,zmin,xmax,ymax,zmax
    for i in range(3):
        j = i+3
        if (bbox[i]<0):
            bbox[i] = 0
        if (bbox[j]>mark_shape[i]):
            bbox[j] = mark_shape[i]
    (xmin,ymin,zmin,xmax,ymax,zmax) = tuple(bbox)

    (x_idxs,y_idxs,z_idxs)=np.where(mark[xmin:xmax,ymin:ymax,zmin:zmax]==0)

    if not use_parent:
        if len(x_idxs) > 0:
            xs = np.asarray(xmin+x_idxs).reshape((len(x_idxs),1))
            ys = np.asarray(ymin+y_idxs).reshape((len(y_idxs),1))
            zs = np.asarray(zmin+z_idxs).reshape((len(z_idxs),1))
            points=np.hstack((xs,ys,zs))

            # 计算所有点到中心点的距离
            center_point = np.array([pos]) # 1*3
            dis_mat = distance_matrix(points, center_point) # M*1

            # 判断距离是否小于半径
            res_idxs = np.where(np.logical_and(MIN_DISTANCE<dis_mat, dis_mat<MAX_DISTANCE))[0]

            if len(res_idxs)>0:
                child_choose = np.random.choice(res_idxs)
                child_pos = (xmin+x_idxs[child_choose], ymin+y_idxs[child_choose], zmin+z_idxs[child_choose])
                return child_pos
            else:
                return None
        else:
            return None
    else:
        if len(x_idxs) > 0:
            xs = np.asarray(xmin+x_idxs-x).reshape((len(x_idxs),1))
            ys = np.asarray(ymin+y_idxs-y).reshape((len(y_idxs),1))
            zs = np.asarray(zmin+z_idxs-z).reshape((len(z_idxs),1))
            points=np.hstack((xs,ys,zs)) # M*3
            parent_vec = np.array([[parent_pos[0]-pos[0]],
                                   [parent_pos[1]-pos[1]],
                                   [parent_pos[2]-pos[2]]]) # 3*1

            # 计算所有点到中心点的距离
            dis_mat = LA.norm(points, axis=1) # M*1
            dis_mat = dis_mat.reshape((dis_mat.shape[0],1))

            # 计算与parent_vec的夹角，保证是向外生长的
            angle_mat = np.matmul(points, parent_vec) # M*1

            # 判断距离是否小于半径
            res_idxs = np.where(np.logical_and(angle_mat<0, dis_mat<MAX_DISTANCE))[0]

            if len(res_idxs)>0:
                child_choose = np.random.choice(res_idxs)
                child_pos = (xmin+x_idxs[child_choose], ymin+y_idxs[child_choose], zmin+z_idxs[child_choose])
                return child_pos
            else:
                return None
        else:
            return None

def setMarkWithSphere(mark, sphere, mark_shape, value, use_bbox=False):
    bbox = list(sphere.calBBox()) # xmin,ymin,zmin,xmax,ymax,zmax
    for i in range(3):
        j = i+3
        if (bbox[i]<0):
            bbox[i] = 0
        if (bbox[j]>mark_shape[i]):
            bbox[j] = mark_shape[i]
    (xmin,ymin,zmin,xmax,ymax,zmax) = tuple(bbox)
    (x_idxs,y_idxs,z_idxs)=np.where(mark[xmin:xmax,ymin:ymax,zmin:zmax]==0)
    # points=img_idxs[:3, xmin+x_idxs, ymin+y_idxs, zmin+z_idxs] # 3*M
    # points=points.T # M*3
    if not use_bbox:
        xs = np.asarray(xmin+x_idxs).reshape((len(x_idxs),1))
        ys = np.asarray(ymin+y_idxs).reshape((len(y_idxs),1))
        zs = np.asarray(zmin+z_idxs).reshape((len(z_idxs),1))
        points=np.hstack((xs,ys,zs))

        sphere_c_mat = np.array([sphere.center_point.toList()]) # 1*3
        # 计算所有点到所有球心的距离
        dis_mat = distance_matrix(points,sphere_c_mat) # M*1

        # 判断距离是否小于半径
        res_idxs = np.where(dis_mat<=sphere.radius)[0]
        mark[xmin+x_idxs[res_idxs], ymin+y_idxs[res_idxs], zmin+z_idxs[res_idxs]] = value
    else:
        mark[xmin+x_idxs, ymin+y_idxs, zmin+z_idxs] = value

def setMarkWithCone(mark, cone, mark_shape, value, use_bbox=False):
    bbox = list(cone.calBBox()) # xmin,ymin,zmin,xmax,ymax,zmax
    for i in range(3):
        j = i+3
        if (bbox[i]<0):
            bbox[i] = 0
        if (bbox[j]>mark_shape[i]):
            bbox[j] = mark_shape[i]
    (xmin,ymin,zmin,xmax,ymax,zmax) = tuple(bbox)

    (x_idxs,y_idxs,z_idxs)=np.where(mark[xmin:xmax,ymin:ymax,zmin:zmax]==0)
    if not use_bbox:
        xs = np.asarray(xmin+x_idxs).reshape((len(x_idxs),1))
        ys = np.asarray(ymin+y_idxs).reshape((len(y_idxs),1))
        zs = np.asarray(zmin+z_idxs).reshape((len(z_idxs),1))
        ns = np.ones((len(z_idxs),1))
        points=np.hstack((xs,ys,zs,ns))

        # 每个圆锥的还原矩阵
        r_min=cone.up_radius
        r_max=cone.bottom_radius
        height=cone.height
        cone_revert_mat = cone.revertMat().T # 4*4

        # 每个椎体还原后坐标
        revert_coor_mat = np.matmul(points, cone_revert_mat) # M*4
        revert_radius_list = LA.norm(revert_coor_mat[:,:2], axis=1) # M

        # Local Indexs
        M = points.shape[0]
        l_idx = np.arange(M) # M (1-dim)
        l_mark = np.ones((M,), dtype=bool)

        # 过滤高度在外部的点
        res_idxs = np.logical_or(revert_coor_mat[l_idx[l_mark],2]<0, revert_coor_mat[l_idx[l_mark],2]>height)
        l_mark[l_idx[l_mark][res_idxs]]=False

        # 过滤半径在外部的点
        res_idxs = revert_radius_list[l_idx[l_mark]]>r_max
        l_mark[l_idx[l_mark][res_idxs]]=False

        # 过滤半径在内部的点
        res_idxs = revert_radius_list[l_idx[l_mark]]<=r_min
        mark[xmin+x_idxs[l_idx[l_mark][res_idxs]], ymin+y_idxs[l_idx[l_mark][res_idxs]], zmin+z_idxs[l_idx[l_mark][res_idxs]]] = value
        l_mark[l_idx[l_mark][res_idxs]]=False

        # 计算剩余
        if r_max>r_min:
            res_idxs = ((r_max-revert_radius_list[l_idx[l_mark]])*height/(r_max-r_min)) >= revert_coor_mat[l_idx[l_mark],2]
            mark[xmin+x_idxs[l_idx[l_mark][res_idxs]], ymin+y_idxs[l_idx[l_mark][res_idxs]], zmin+z_idxs[l_idx[l_mark][res_idxs]]] = value
            l_mark[l_idx[l_mark][res_idxs]]=False
    else:
        mark[xmin+x_idxs, ymin+y_idxs, zmin+z_idxs] = value

def simulate3DTree():
    MAX_TREE_DEPTH = 4
    MAX_RADIUS = 6
    SAFE_DISTANCE = MAX_RADIUS + 2

    MAX_DISTANCE = 16

    mark_shape = (251,251,251)

    # Init space
    mark = np.zeros(mark_shape, dtype=np.uint8)
    mark_shape = mark.shape
    node_count = 0

    # Create root node
    root_r = getChildRadius(0,MAX_TREE_DEPTH)
    root_pos = (125,125,125)
    node_count += 1
    root_node = Vertex(node_count,0,root_pos[0],root_pos[1],root_pos[2],root_r,-1)
    setMarkWithSphere(mark, Sphere(Point3D(*root_node.pos), root_node.r), mark_shape, 255)
    # setMarkWithSphere(mark, Sphere(Point3D(*root_node.pos), root_node.r + SAFE_DISTANCE), mark_shape, 1)

    # Creante dequeue and list to contain result
    dq = deque([(root_node, 0)]) # 第二项表示node节点的depth
    nodes = {}
    graph = {}

    while len(dq):
        root_node = dq[0][0]
        root_depth = dq[0][1]
        dq.popleft()

        # Add to nodes and graph
        v1 = root_node.idx
        v2 = root_node.p_idx
        if root_node.idx not in nodes:
            nodes[root_node.idx] = root_node
        if v1>0 and v2>0:
            if not v1 in graph:
                graph[v1] = set([v2])
            else:
                graph[v1].add(v2)

            if not v2 in graph:
                graph[v2] = set([v1])
            else:
                graph[v2].add(v1)

        if root_depth<MAX_TREE_DEPTH:
            # Get children number
            if root_node.idx==1: # 根节点与其他节点单独处理
                child_num = 4
                mask = np.array([[1,1,1],
                                 [-1,1,1],
                                 [1,1,-1],
                                 [-1,1,-1]])
                for i in range(4):
                    # 获取分支半径和长度
                    child_r = getChildRadius(root_depth+1,MAX_TREE_DEPTH)
                    child_length = getChildLength(root_depth+1,MAX_TREE_DEPTH)

                    #theta_z = np.random.uniform(30,60)
                    theta_y = 45
                    #A = rotation_matrix(theta_z/180*np.math.pi, [0,0,1])
                    B = rotation_matrix(-theta_y/180*np.math.pi, [0,1,0])
                    # rot_mat = np.matmul(A,B)
                    p0 = np.array([[child_length],[0],[0],[1]])
                    p1 = np.matmul(B, p0)
                    child_pos = (int(p1[0]*mask[i][0]+root_node.pos[0]), \
                                 int(p1[1]*mask[i][1]+root_node.pos[1]), \
                                 int(p1[2]*mask[i][2]+root_node.pos[2]))
                    if ImageUtils.bboxCheck3D(child_pos[0], child_pos[1], child_pos[2], child_r, mark_shape):
                        node_count += 1
                        #print('parent', root_node.idx, 'id', node_count, 'pos', child_pos, 'depth', root_depth+1)
                        child_node = Vertex(node_count, 0, child_pos[0], child_pos[1], child_pos[2], child_r, root_node.idx)
                        # 绘制
                        setMarkWithSphere(mark, Sphere(Point3D(*child_node.pos), child_node.r), mark_shape, 255)
                        setMarkWithCone(mark, Cone(Point3D(*root_node.pos), root_node.r, \
                                                   Point3D(*child_node.pos), child_node.r), mark_shape, 255)

                        # Add to dequeue
                        dq.append((child_node, root_depth+1))
            else:

                child_num = getRandChildNumber()
                child_angles_range = Draw3DTools.sliceRange(0, 360, child_num)

                for i in range(child_num):

                    # 获取分支半径和长度
                    child_r = getChildRadius(root_depth+1,MAX_TREE_DEPTH)
                    child_length = getChildLength(root_depth+1,MAX_TREE_DEPTH)

                    # 获取生长角度
                    if child_num==1:
                        theta_z = np.random.uniform(0,360)
                        theta_y = np.random.uniform(60,90)
                    else:
                        theta_z = np.random.uniform(child_angles_range[i][0],child_angles_range[i][1])
                        theta_y = np.random.uniform(30,70)

                    A = rotation_matrix(theta_z/180*np.math.pi, [0,0,1])
                    B = rotation_matrix(-theta_y/180*np.math.pi, [0,1,0])
                    rot_mat = np.matmul(A,B)
                    p0 = np.array([[child_length],[0],[0],[1]])
                    p1 = np.matmul(rot_mat, p0)

                    grand_node = nodes[root_node.p_idx] # root节点的父节点
                    p_a = Point3D(0,0,0)
                    p_c = Point3D(root_node.pos[0]-grand_node.pos[0], \
                                  root_node.pos[1]-grand_node.pos[1], \
                                  root_node.pos[2]-grand_node.pos[2])
                    p_b = p_a.medianWithPoint(p_c)
                    v1=np.array([[p_a.x, p_b.x, p_c.x],
                                 [p_a.y, p_b.y, p_c.y],
                                 [p_a.z, p_b.z, p_c.z],
                                 [   1,    1,    1]])
                    Dis=p_a.distanceWithPoint(p_c)
                    v0=np.array([[0,     0,   0],
                                 [0,     0,   0],
                                 [-Dis, -Dis/2, 0],
                                 [1,     1,  1]])
                    rev_mat = superimposition_matrix(v0,v1)
                    p2 = np.matmul(rev_mat, p1)
                    child_pos = (int(p2[0]+grand_node.pos[0]), int(p2[1]+grand_node.pos[1]), int(p2[2]+grand_node.pos[2]))
                    if ImageUtils.bboxCheck3D(child_pos[0], child_pos[1], child_pos[2], child_r, mark_shape):
                        node_count += 1
                        #print('parent', root_node.idx, 'id', node_count, 'pos', child_pos, 'depth', root_depth+1)
                        child_node = Vertex(node_count, 0, child_pos[0], child_pos[1], child_pos[2], child_r, root_node.idx)
                        # 绘制
                        setMarkWithSphere(mark, Sphere(Point3D(*child_node.pos), child_node.r), mark_shape, 255)
                        setMarkWithCone(mark, Cone(Point3D(*root_node.pos), root_node.r, \
                                                   Point3D(*child_node.pos), child_node.r), mark_shape, 255)

                        # Add to dequeue
                        dq.append((child_node, root_depth+1))

    mark = np.where(mark==255, 255, 0).astype(np.uint8)
    mark = np.swapaxes(mark, 0, 2)
    return mark, nodes, graph



import unittest
from pyneval.model.swc_node import SwcTree
from pyneval.metric.diadem_metric import remove_spurs, generate_node_weights, g_weight_dict
from pyneval.metric.utils.bin_utils import convert_to_binarytree


def test_get_bintree():
    goldtree = SwcTree()
    goldtree.load("D:\gitProject\mine\PyNeval\\test\data_example\\test\\ExampleTest.swc")
    bintree = convert_to_binarytree(goldtree)
    return bintree


class TestStringMethods(unittest.TestCase):

    def test_remove_spurs(self):
        bintree = test_get_bintree()
        spur_set = remove_spurs(bintree, 1.0)
        for item in spur_set:
            print(item.data.get_id())

    def test_weight(self):
        bin_tree = test_get_bintree()
        generate_node_weights(bin_tree, set())
        for key in g_weight_dict.keys():
            print("key = {}, num = {}".format(
                key.data._id, g_weight_dict[key]
            ))

    def testTra(self):
        bin_tree = test_get_bintree()
        bin_tree_list = bin_tree.get_node_list()
        for node in bin_tree_list:
            print(node.data.get_id())
            if node.data.parent_trajectory is not None:
                print(node.data.parent_trajectory._pos)
            if node.data.left_trajectory is not None:
                print(node.data.left_trajectory._pos)
            if node.data.right_trajectory is not None:
                print(node.data.right_trajectory._pos)
            print("------------------------")

if __name__ == "__main__":
    unittest.main()


import os

from pyneval.io.read_swc import read_swc_trees
from pyneval.io import swc_writer
from pyneval.model.swc_node import SwcTree
import random


def get_float_random(down, up):
    if down > up:
        return False
    res = random.random() * (up-down) + down
    return res


def swc_random_delete(swc_tree, move_percentage=None, move_num=None):
    '''
    :param move_percentage:
    :param swc_tree:
    :return: randomly remove a node from the original tree
    '''

    swc_node_list = swc_tree.get_node_list()

    # calculate which nodes to move_5
    node_to_move = [i for i in range(1, len(swc_node_list))]
    if move_num is not None:
        node_to_move = node_to_move[:move_num]
    elif move_percentage is not None:
        node_to_move = node_to_move[:int(move_percentage*len(swc_node_list))]
    else:
        return False

    res_tree = swc_tree.get_copy()
    swc_node_list = res_tree.get_node_list()

    for remove_id in node_to_move:
        remove_node = swc_node_list[remove_id]
        # for son in remove_node.children:
        #     son.parent = res_tree.root()
        res_tree.remove_node(remove_node.parent, remove_node)
        # remove_node.parent.remove_child(remove_node)
        remove_node.parent = None
    res_tree.get_node_list(update=True)
    # swc_save(res_tree, "D:\gitProject\mine\PyNeval\output\\build_out.swc")
    return res_tree


def swc_random_move(swc_tree, move_percentage=None, move_num=None, move_range=1.0, tendency=(1, 1, 1)):
    '''
    :param swc_tree: standard swc tree
    :param percentage: percentage of nodes to move (range:[0,1])
    :param range:stard range is the averange length of edge.
    :param tendency: a tuple to change the direction of the movement
    :return: modified swc tree(different object from the input one)
    '''
    swc_node_list = swc_tree.get_node_list()

    # calculate move_5 base
    tot_rad = 0.0
    for node in swc_node_list:
        if node.is_virtual():
            continue
        tot_rad += node.radius()
    move_base = tot_rad / float(len(swc_node_list) - 1) * move_range

    # calculate which nodes to volume_move
    node_to_move = [i for i in range(1, len(swc_node_list))]
    random.shuffle(node_to_move)
    if move_num is not None:
        node_to_move = node_to_move[:move_num]
    elif move_percentage is not None:
        node_to_move = node_to_move[:int(move_percentage*len(swc_node_list))]
    else:
        return False

    res_tree = swc_tree.get_copy()
    res_swc_list = res_tree.get_node_list()
    for node_id in node_to_move:
        # calculate move_5 length
        dx, dy, dz = get_float_random(-1, 1), get_float_random(-1, 1), get_float_random(-1, 1)
        dx, dy, dz = dx * move_base, dy * move_base, dz * move_base
        dx, dy, dz = dx * tendency[0], dy * tendency[1], dz * tendency[2]

        # print("dx = {} dy = {} dz = {}".format(dx, dy, dz))
        res_swc_list[node_id].set_x(res_swc_list[node_id].get_x() + dx)
        res_swc_list[node_id].set_y(res_swc_list[node_id].get_y() + dy)
        res_swc_list[node_id].set_z(res_swc_list[node_id].get_z() + dz)

    # debug
    # swc_save(res_tree, "D:\gitProject\mine\PyNeval\output\\build_out.swc")
    return res_tree


def swc_random_link(swc_tree, move_percentage=None, move_num=None):
    '''
    :param swc_tree: standard swc tree
    :param percentage: percentage of nodes to volume_move (range:[0,1])
    :param range:stard range is the averange length of edge.
    :param tendency: a tuple to change the direction of the movement
    :return: modified swc tree(different object from the input one)
    '''
    branch_list = swc_tree.get_branch_swc_list()

    # calculate which nodes to volume_move
    node_to_modify = [i for i in range(1, len(branch_list))]
    random.shuffle(node_to_modify)
    if move_num is not None:
        node_to_modify = node_to_modify[:move_num]
    elif move_percentage is not None:
        node_to_modify = node_to_modify[:int(move_percentage*len(branch_list))]
    else:
        return False

    res_tree = swc_tree.get_copy()
    res_swc_list = res_tree.get_branch_swc_list()

    for node_id in node_to_modify:
        pa = res_swc_list[node_id]
        rm_c_id = -1
        if len(res_swc_list[node_id].children) >= 1:
            rm_c_id = random.randint(0, len(res_swc_list[node_id].children) - 1)
        if rm_c_id == -1:
            continue
        son = res_swc_list[node_id].children[rm_c_id]

        res_tree.unlink_child(son)

        res_tree.link_child(pa.parent, son)
    return res_tree


def generate_data():
    tree_name_dict = {}
    gold_trees = read_swc_trees(swc_file_paths="../../data/example_selected",
                                tree_name_dict=tree_name_dict)
    iter_num = 10
    move_num = None
    move_range = 2.0
    move_tendency = (1, 1, 1)
    output_dir = "../../output/random_data"
    # decide the origin swc tree
    for gold_tree in gold_trees:
        # decide how much modes to change
        tree_name = tree_name_dict[gold_tree]
        tree_dir = os.path.join(output_dir, tree_name[:-4])
        if not os.path.exists(tree_dir):
            os.mkdir(tree_dir)
        for move_percentage in range(1, 11):
            # take random for gen_num times
            percent_dir = os.path.join(tree_dir, "{:03d}".format(move_percentage*10))
            if not os.path.exists(percent_dir):
                os.mkdir(percent_dir)
            for it in range(iter_num):
                test_swc = swc_random_move(swc_tree=gold_tree,
                                           move_percentage=0.1 * move_percentage,
                                           move_num=None,
                                           move_range=move_range,
                                           tendency=move_tendency)
                # test_swc = swc_random_delete(swc_tree=gold_tree,
                #                              move_percentage=0.1*move_percentage,
                #                              move_num=None)
                # test_swc = swc_random_link(swc_tree=gold_tree,
                #                            move_percentage=0.1 * move_percentage,
                #                            move_num=None)
                file_name = os.path.join(percent_dir, "move_{:02d}.swc".format(it))
                swc_writer.swc_save(swc_tree=test_swc, out_path=file_name)


if __name__ == "__main__":
    generate_data()



import unittest
import sys
from pyneval.metric.ssd_metric import ssd_metric
from pyneval.model.swc_node import SwcTree, SwcNode
from pyneval.io.read_json import read_json


class SSDMetricTest(unittest.TestCase):
    def test_ssd_1(self):
        sys.setrecursionlimit(1000000)
        gold_swc_tree = SwcTree()
        test_swc_tree = SwcTree()
        test_swc_tree.load("..\\..\\..\\data\\test_data\\ssd_data\\test\\fake_data1.swc")
        gold_swc_tree.load("..\\..\\..\\data\\test_data\\ssd_data\\gold\\fake_data1.swc")
        config = read_json("..\\..\\..\\config\\ssd_metric.json")
        config['threshold_dis'] = 10
        ssd_result = ssd_metric(test_swc_tree=gold_swc_tree, gold_swc_tree=test_swc_tree, config=config)
        ans = [1.0920186279062936, 0.92, 0.8666666666666667]
        for i in range(3):
            print(ssd_result[i])
            self.assertEqual(ssd_result[i], ans[i])


if __name__ == '__main__':
    unittest.main()


import glob
import os
import numpy as np
from scipy import ndimage
from PIL import Image
import tensorflow as tf
from LabelDefinition import class_d_label
import cv2
import TiffFile
import pims

def readImageAndLabelFromFolders(paths,
                                 number=None,
                                 resized=False,
                                 new_shape=None):
    '''readImageAndLabelFromFolders
    Args:
        paths (list): List of path.
        number (list): Number of data.
        resized (bool): Whether resize.
        new_shape (2d tuple): New shape if resize.
    Returns:
        images (ndarray): Numpy array 2d image matrix.
        labels (ndarray): Numpy array of int.
        shapes (ndarray): Numpy array of 3d tuple.
    '''
    all_images = []
    all_labels = []
    all_shapes = []
    shapeset = set()

    for idx, p in enumerate(paths):
        images, labels, shapes, shape_set = readImageAndLabel(p,
                                                            number=None,
                                                            resized=False,
                                                            new_shape=None)
        print(shape_set)
        if len(shape_set)!=1:
            print('More than two image shapes, exit...')
            quit(code=-1)

        all_images.append(images)
        all_labels.append(labels)
        all_shapes.append(shapes)
        shapeset.add(shape_set.pop())

    print(shapeset)
    if len(shapeset)!=1:
        print('More than two image shapes, exit...')
        quit(code=-1)
    actual_shape = shapeset.pop()

    ret_images = np.array([], np.uint8).reshape(0, actual_shape[0], actual_shape[1], actual_shape[2])
    ret_labels = np.array([], np.uint32)
    ret_shapes = np.array([], np.int64).reshape(0,3)

    # import pdb
    # pdb.set_trace()
    for a,b,c in zip(all_images, all_labels, all_shapes):
        ret_images = np.concatenate((ret_images, a), axis=0)
        ret_labels = np.concatenate((ret_labels, b), axis=0)
        ret_shapes = np.concatenate((ret_shapes, c), axis=0)

    return ret_images, ret_labels, ret_shapes, shapeset

def readImageAndLabelFromFolders3D(paths,
                                 number=None):
    '''readImageAndLabelFromFolders
    Args:
        paths (list): List of path.
        number (list): Number of data.
        resized (bool): Whether resize.
        new_shape (2d tuple): New shape if resize.
    Returns:
        images (ndarray): Numpy array 2d image matrix.
        labels (ndarray): Numpy array of int.
        shapes (ndarray): Numpy array of 3d tuple.
    '''
    all_images = []
    all_labels = []
    all_shapes = []
    shapeset = set()

    for idx, p in enumerate(paths):
        images, labels, shapes, shape_set = readImageAndLabel3D(p,number)
        print(shape_set)
        if len(shape_set)!=1:
            print('More than two image shapes, exit...')
            quit(code=-1)

        all_images.append(images)
        all_labels.append(labels)
        all_shapes.append(shapes)
        shapeset.add(shape_set.pop())

    print(shapeset)
    if len(shapeset)!=1:
        print('More than two image shapes, exit...')
        quit(code=-1)
    actual_shape = shapeset.pop()

    ret_images = np.array([], np.uint8).reshape(0,
                                                actual_shape[0],
                                                actual_shape[1],
                                                actual_shape[2],
                                                actual_shape[3])
    ret_labels = np.array([], np.uint32)
    ret_shapes = np.array([], np.int64).reshape(0,4)

    for a,b,c in zip(all_images, all_labels, all_shapes):
        ret_images = np.concatenate((ret_images, a), axis=0)
        ret_labels = np.concatenate((ret_labels, b), axis=0)
        ret_shapes = np.concatenate((ret_shapes, c), axis=0)

    return ret_images, ret_labels, ret_shapes, shapeset

def readImageAndLabel(path, number=None,
                               resized=False, new_shape=None):
    '''read images and return image and label array
    '''

    # directories = glob.glob(os.path.join(path, '*'))
    # class_names = [os.path.basename(directory) for directory in directories]
    # class_names.sort()
    # num_classes = len(class_names)

    file_paths = glob.glob(os.path.join(path, '*/*'))
    # file_paths = sorted(file_paths,
    #        key=lambda filename:os.path.basename(filename).split('.')[0])
    np.random.shuffle(file_paths)

    images = []
    labels = []
    shapes = []
    shape_set = set()
    for file_name in file_paths[:number]:
        im = Image.open(file_name)
        if resized:
            im = im.resize(new_shape)

        im = np.asarray(im, np.uint8)
        if len(im.shape) == 2:
            im = im.reshape(im.shape[0], im.shape[1], 1)
        images.append(im)

        im_shape = im.shape
        shapes.append(im_shape)  # tuple of 3
        if im_shape not in shape_set:
            shape_set.add(im_shape)
        # image_name = os.path.basename(file_name).split('.')[0]

        class_name = os.path.basename(os.path.dirname(file_name))
        label_num = class_d_label[class_name]
        labels.append(np.asarray(label_num, np.uint32))

    images = np.array(images)
    labels = np.array(labels)
    shapes = np.array(shapes)
    print('shape set: ', shape_set)
    return images, labels, shapes, shape_set


def readImageAndLabel3D(path, number=None):
    '''read images and return image and label array 3D
    '''
    file_paths = glob.glob(os.path.join(path, '*/*'))
    np.random.shuffle(file_paths)

    images = []
    labels = []
    shapes = []
    shape_set = set()
    for file_name in file_paths[:number]:
        im = TiffFile.imread(file_name)
        # im = pims.open(file_name)
        # im = np.asarray(im, np.uint8)

        if len(im.shape) == 3:
            im = im.reshape(im.shape[0], im.shape[1], im.shape[2], 1)
        images.append(im)

        im_shape = im.shape
        shapes.append(im_shape)  # tuple of 4
        if im_shape not in shape_set:
            shape_set.add(im_shape)

        class_name = os.path.basename(os.path.dirname(file_name))
        label_num = class_d_label[class_name]
        labels.append(np.asarray(label_num, np.uint32))

    images = np.array(images)
    labels = np.array(labels)
    shapes = np.array(shapes)
    print('shape set: ', shape_set)
    return images, labels, shapes, shape_set

def inputPipeline(filenames,
                      batch_size,
                      num_epochs,
                      num_threads,
                      old_shape,
                      do_train,
                      normalize=False,
                      resize_type=0,
                      new_shape=None):
    ''' The input pipeline for TensorFlow graph
    '''
    filename_queue = tf.train.string_input_producer(
                     filenames, num_epochs=num_epochs, shuffle=True)
    if len(old_shape)==3:
        print('*'*20,'2D inputPipeline', '*'*20)
        example, label = _readImagesFromTFRecordsFile(filename_queue,
                                                               old_shape,
                                                               normalize,
                                                               do_train, # train vs eval, diff preprocess
                                                               resize_type,
                                                               new_shape)
    else:
        print('*'*20,'3D inputPipeline', '*'*20)
        example, label = _readImagesFromTFRecordsFile3D(filename_queue,
                                                               old_shape,
                                                               normalize,
                                                               do_train, # train vs eval, diff preprocess
                                                               resize_type,
                                                               new_shape)
    # min_after_dequeue defines how big a buffer we will randomly sample
    #   from -- bigger means better shuffling but slower start up and more
    #   memory used.
    # capacity must be larger than min_after_dequeue and the amount larger
    #   determines the maximum we will prefetch.  Recommendation:
    #   min_after_dequeue + (num_threads + a small safety margin) * batch_size
    min_after_dequeue = 1000
    capacity = min_after_dequeue + (num_threads+3) * batch_size
    example_batch, label_batch =  \
        tf.train.shuffle_batch([example, label],
                               batch_size=batch_size,
                               num_threads=num_threads,
                               capacity=capacity,
                               # Ensures a minimum amount of shuffling
                               min_after_dequeue=min_after_dequeue,
                               name='batching_shuffing')
    return example_batch, label_batch

def _readImagesFromTFRecordsFile(filename_queue,
                                          old_shape,
                                          normalize,
                                          do_train,
                                          resize_type,
                                          new_shape):

    reader = tf.TFRecordReader()
    key, serialized_example = reader.read(filename_queue)

    features = tf.parse_single_example(serialized_example,
                                       features={'image_raw': tf.FixedLenFeature([], tf.string),
                                                 'label': tf.FixedLenFeature([], tf.int64)
                                                })

    image = tf.decode_raw(features['image_raw'], tf.uint8)
    image = tf.cast(image, tf.float32)
    label = tf.cast(features['label'], tf.int32)
    image = tf.reshape(image, old_shape)
    image.set_shape(old_shape)
    print('original image shape: ', image.get_shape())

    if resize_type == 1:  # resize_type default 0, do nothing
        image = tf.image.rgb_to_grayscale(image)
        image = tf.image.crop_to_bounding_box(image, 0, 55, 223, 223)
        image = tf.image.resize_images(image, [new_shape[0], new_shape[1]])
        image = tf.reshape(image, new_shape)
    elif resize_type == 2:
        ''' Random crop 40by40 to 32by32
            Image is default gray
        '''
        # offset_height = np.random.randint(0, 9)
        # offset_width = np.random.randint(0, 9)
        # image = tf.image.crop_to_bounding_box(image,
        #                                       offset_height,
        #                                       offset_width,
        #                                       new_shape[0],
        #                                       new_shape[1])
        image = tf.image.crop_to_bounding_box(image,
                                              0,
                                              0,
                                              new_shape[0],
                                              new_shape[1])
        # print 'TensorFlow random crop'
        # image = tf.random_crop(image, (32, 32, 1))
        image = tf.reshape(image, new_shape)
    elif resize_type == 3:
        # image = tf.image.resize_images(image, [new_shape[0], new_shape[1]])
        # image = tf.image.crop_to_bounding_box(image,
        #                                       7,
        #                                       0,
        #                                       new_shape[0],
        #                                       new_shape[1])

        print( 'TensorFlow random crop')
        image = tf.random_crop(image, (32, 32, 1))
        image = tf.image.random_flip_up_down(image)
        image = tf.image.random_flip_left_right(image)
        if do_train:
            image = tf.image.random_brightness(image,max_delta=63)
            image = tf.image.random_contrast(image,lower=0.2, upper=1.8)
        image = tf.reshape(image, new_shape)

    print('new image shape: ', image.get_shape())
    print('label shape: ', label.get_shape())

    processed_example = image
    return processed_example, label

def preprocess(array):
    # swap_axises = tf.constant([0,1,2])
    # c = tf.random_shuffle(c)
    return ndimage.rotate(array, 45)

def _readImagesFromTFRecordsFile3D(filename_queue,
                                          old_shape,
                                          normalize,
                                          do_train,
                                          resize_type,
                                          new_shape):

    reader = tf.TFRecordReader()
    key, serialized_example = reader.read(filename_queue)

    features = tf.parse_single_example(serialized_example,
                                       features={'image_raw': tf.FixedLenFeature([], tf.string),
                                                 'label': tf.FixedLenFeature([], tf.int64)
                                                })

    image = tf.decode_raw(features['image_raw'], tf.uint8)
    image = tf.cast(image, tf.float32)
    label = tf.cast(features['label'], tf.int32)
    image = tf.reshape(image, old_shape)
    image.set_shape(old_shape)
    print('original image shape:', image.get_shape())

    ## Default 0, just resize
    if resize_type == 1:  # Just resize
        ## Don't crop, augment and resize
        swap_axises = tf.constant([0,1,2])
        swap_axises = tf.random_shuffle(swap_axises)
        image = tf.transpose(image, perm=[swap_axises[0],
                                          swap_axises[1],
                                          swap_axises[2],
                                          3])
        image = tf.reshape(image, new_shape)
    elif resize_type == 2:
        ## Random crop , flip
        image = tf.random_crop(image, tuple(new_shape))
        swap_axises = tf.constant([0,1,2])
        swap_axises = tf.random_shuffle(swap_axises)
        image = tf.transpose(image, perm=[swap_axises[0],
                                          swap_axises[1],
                                          swap_axises[2],
                                          3])
        image = tf.reshape(image, new_shape)
        image.set_shape(new_shape)
    elif resize_type == 3:
        ## Random crop, flip and adjust contrast
        image = tf.random_crop(image, tuple(new_shape))
        swap_axises = tf.constant([0,1,2])
        swap_axises = tf.random_shuffle(swap_axises)
        image = tf.transpose(image, perm=[swap_axises[0],
                                          swap_axises[1],
                                          swap_axises[2],
                                          3])
        # noise = tf.random_uniform(tuple(new_shape),minval=0,maxval=None,dtype=tf.float32)
        image = tf.image.random_contrast(image,lower=0.3, upper=1.5)
        image = tf.reshape(image, new_shape)
        image.set_shape(new_shape)

    processed_example = image
    return processed_example, label

def getLabelClassMap(base_dir):
    directories = glob.glob(os.path.join(base_dir, '*'))
    class_names = [os.path.basename(directory) for directory in directories]
    class_names.sort()
    label_d_class = dict(zip(range(len(class_names)), class_names))
    class_d_label = dict(zip(class_names, range(len(class_names))))
    return label_d_class, class_d_label



import unittest
import numpy as np
from pyneval.metric.utils.edge_match_utils import get_idedge_dict, get_edge_rtree, get_nearby_edges
from pyneval.model.swc_node import SwcTree,SwcNode
from pyneval.model.euclidean_point import EuclideanPoint,Line
from rtree import index


# 获取线段的bounding box
def get_bounds(point_a, point_b):
    point_a = np.array(point_a._pos)
    point_b = np.array(point_b._pos)
    # res = np.where(point_a>point_b,point_b,point_a).tolist() + np.where(point_a>point_b,point_a,point_b).tolist()
    return tuple(point_a.tolist() + point_b.tolist())


class TestStringMethods(unittest.TestCase):

    def test_upper(self):
        self.assertEqual('foo'.upper(), 'FOO')

    def test_isupper(self):
        self.assertTrue('FOO'.isupper())
        self.assertFalse('Foo'.isupper())

    def test_split(self):
        s = 'hello world'
        self.assertEqual(s.split(), ['hello', 'world'])
        # check that s.split fails when the separator is not a string
        with self.assertRaises(TypeError):
            s.split(2)

    def test_rtree(self):
        swctree = SwcTree()
        swctree.load("D:\gitProject\mine\PyNeval\\test\data_example\\test\\30_18_10_test.swc")
        rtree = get_edge_rtree(swctree)
        id_edge_dict = get_idedge_dict(swctree)
        point = SwcNode(center=[22.5822, 172.856, 300.413])
        (line_tuple, dis) = get_nearby_edges(rtree, point, id_edge_dict, point.radius())[0]

        print(point.distance(Line(coords=[[15.3249, 130.821, 327.012],[19.8495, 132.384, 323.395]])))
        print("dis = {}, line:{}, {}".format(
            dis, line_tuple[0]._pos,line_tuple[1]._pos
        ))


if __name__ == "__main__":
    unittest.main()


import os
import codecs

class Vertex(object):
    '''Define a Vertex class represent a node in SWC structure'''
    def __init__(self, idx, itype, x, y, z, r, p_idx):
        """ Constructor
            idx: id
            itype: node type
            x,y,z: 3d position
            r: radius
            p_idx: parent index
        """
        self.idx = idx
        self.itype = itype
        self.pos = [x,y,z]
        self.r = r
        self.p_idx = p_idx

    def getChilds(self, graph):
        ''' Get children's id of this node.
            Input:
                graph(dict<int,set<int>>)
            Output:
                list
        '''
        neighbors = list(graph[self.idx])
        return list(filter(lambda x:x!=self.p_idx, neighbors))

    def __str__(self):
        """ Convert Vertex to string
            when print(Vertex()) is called, this function will be called
        """
        return 'id{1}:{0}\n'.format(self.idx,type(self.idx)) \
               + 'itype{1}:{0}\n'.format(self.itype,type(self.itype)) \
               + 'pos{1}:{0}\n'.format(self.pos,type(self.pos)) \
               + 'r{1}:{0}\n'.format(self.r,type(self.r)) \
               + 'pid{1}:{0}\n'.format(self.p_idx,type(self.p_idx))

def extractSWC(file_path):
    """ Extract the sturcture of swc file
        Input:
            file_path:str path to swc file
        Output:
            nodes:dict vertex list information
            graph:dict the topo of each node
    """
    nodes = {} # id:Vertex
    graph = {} # id:Set(id)

    f = open(file_path)
    # f = codecs.open(file_path, "r",encoding='utf-8', errors='ignore')

    count = 0
    for line in f:
        if not line.startswith('#'):
            args = line.split(' ')
            args = list(filter(None,args)) # *Remove all empty string

            if len(args) == 7:
                if args[-1].endswith(os.linesep): # *Remove line separater
                    args[-1] = args[-1][:-len(os.linesep)]

                    args = list(map(float,args)) # *Convert data type to float
                    args[0] = int(args[0])
                    args[1] = int(args[1])
                    args[-1] = int(args[-1])

                    v1 = args[0]
                    v2 = args[-1]

                    vertex = Vertex(*args[:]) # *Construct a class using list as parameter
                    if vertex.idx not in nodes: # Add vertex
                        nodes[vertex.idx]  = vertex

                    if v2 > 0 and v1 > 0: # Add edge to graph

                        if not v1 in graph:
                            graph[v1] = set([v2])
                        else:
                            graph[v1].add(v2)

                        if not v2 in graph:
                            graph[v2] = set([v1])
                        else:
                            graph[v2].add(v1)

                    else:
                        pass

    return nodes, graph



import tflib as lib

import numpy as np
import tensorflow as tf

_default_weightnorm = False
def enable_default_weightnorm():
    global _default_weightnorm
    _default_weightnorm = True

def Conv1D(name, input_dim, output_dim, filter_size, inputs, he_init=True, mask_type=None, stride=1, weightnorm=None, biases=True, gain=1.):
    """
    inputs: tensor of shape (batch size, num channels, width)
    mask_type: one of None, 'a', 'b'

    returns: tensor of shape (batch size, num channels, width)
    """
    with tf.name_scope(name) as scope:

        if mask_type is not None:
            mask_type, mask_n_channels = mask_type

            mask = np.ones(
                (filter_size, input_dim, output_dim), 
                dtype='float32'
            )
            center = filter_size // 2

            # Mask out future locations
            # filter shape is (width, input channels, output channels)
            mask[center+1:, :, :] = 0.

            # Mask out future channels
            for i in range(mask_n_channels):
                for j in range(mask_n_channels):
                    if (mask_type=='a' and i >= j) or (mask_type=='b' and i > j):
                        mask[
                            center,
                            i::mask_n_channels,
                            j::mask_n_channels
                        ] = 0.


        def uniform(stdev, size):
            return np.random.uniform(
                low=-stdev * np.sqrt(3),
                high=stdev * np.sqrt(3),
                size=size
            ).astype('float32')

        fan_in = input_dim * filter_size
        fan_out = output_dim * filter_size / stride

        if mask_type is not None: # only approximately correct
            fan_in /= 2.
            fan_out /= 2.

        if he_init:
            filters_stdev = np.sqrt(4./(fan_in+fan_out))
        else: # Normalized init (Glorot & Bengio)
            filters_stdev = np.sqrt(2./(fan_in+fan_out))

        filter_values = uniform(
            filters_stdev,
            (filter_size, input_dim, output_dim)
        )
        # print "WARNING IGNORING GAIN"
        filter_values *= gain

        filters = lib.param(name+'.Filters', filter_values)

        if weightnorm==None:
            weightnorm = _default_weightnorm
        if weightnorm:
            norm_values = np.sqrt(np.sum(np.square(filter_values), axis=(0,1)))
            target_norms = lib.param(
                name + '.g',
                norm_values
            )
            with tf.name_scope('weightnorm') as scope:
                norms = tf.sqrt(tf.reduce_sum(tf.square(filters), reduction_indices=[0,1]))
                filters = filters * (target_norms / norms)

        if mask_type is not None:
            with tf.name_scope('filter_mask'):
                filters = filters * mask

        result = tf.nn.conv1d(
            value=inputs, 
            filters=filters, 
            stride=stride,
            padding='SAME',
            data_format='NCHW'
        )

        if biases:
            _biases = lib.param(
                name+'.Biases',
                np.zeros([output_dim], dtype='float32')
            )

            # result = result + _biases

            result = tf.expand_dims(result, 3)
            result = tf.nn.bias_add(result, _biases, data_format='NCHW')
            result = tf.squeeze(result)

        return result



# From https://github.com/openai/improved-gan/blob/master/inception_score/model.py
# Code derived from tensorflow/tensorflow/models/image/imagenet/classify_image.py
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os.path
import sys
import tarfile

import numpy as np
from six.moves import urllib
import tensorflow as tf
import glob
import scipy.misc
import math
import sys

MODEL_DIR = '/tmp/imagenet'
DATA_URL = 'http://download.tensorflow.org/models/image/imagenet/inception-2015-12-05.tgz'
softmax = None

# Call this function with list of images. Each of elements should be a 
# numpy array with values ranging from 0 to 255.
def get_inception_score(images, splits=10):
  assert(type(images) == list)
  assert(type(images[0]) == np.ndarray)
  assert(len(images[0].shape) == 3)
  assert(np.max(images[0]) > 10)
  assert(np.min(images[0]) >= 0.0)
  inps = []
  for img in images:
    img = img.astype(np.float32)
    inps.append(np.expand_dims(img, 0))
  bs = 100
  with tf.Session() as sess:
    preds = []
    n_batches = int(math.ceil(float(len(inps)) / float(bs)))
    for i in range(n_batches):
        # sys.stdout.write(".")
        # sys.stdout.flush()
        inp = inps[(i * bs):min((i + 1) * bs, len(inps))]
        inp = np.concatenate(inp, 0)
        pred = sess.run(softmax, {'ExpandDims:0': inp})
        preds.append(pred)
    preds = np.concatenate(preds, 0)
    scores = []
    for i in range(splits):
      part = preds[(i * preds.shape[0] // splits):((i + 1) * preds.shape[0] // splits), :]
      kl = part * (np.log(part) - np.log(np.expand_dims(np.mean(part, 0), 0)))
      kl = np.mean(np.sum(kl, 1))
      scores.append(np.exp(kl))
    return np.mean(scores), np.std(scores)

# This function is called automatically.
def _init_inception():
  global softmax
  if not os.path.exists(MODEL_DIR):
    os.makedirs(MODEL_DIR)
  filename = DATA_URL.split('/')[-1]
  filepath = os.path.join(MODEL_DIR, filename)
  if not os.path.exists(filepath):
    def _progress(count, block_size, total_size):
      sys.stdout.write('\r>> Downloading %s %.1f%%' % (
          filename, float(count * block_size) / float(total_size) * 100.0))
      sys.stdout.flush()
    filepath, _ = urllib.request.urlretrieve(DATA_URL, filepath, _progress)
    print()
    statinfo = os.stat(filepath)
    print('Succesfully downloaded', filename, statinfo.st_size, 'bytes.')
  tarfile.open(filepath, 'r:gz').extractall(MODEL_DIR)
  with tf.gfile.FastGFile(os.path.join(
      MODEL_DIR, 'classify_image_graph_def.pb'), 'rb') as f:
    graph_def = tf.GraphDef()
    graph_def.ParseFromString(f.read())
    _ = tf.import_graph_def(graph_def, name='')
  # Works with an arbitrary minibatch size.
  with tf.Session() as sess:
    pool3 = sess.graph.get_tensor_by_name('pool_3:0')
    ops = pool3.graph.get_operations()
    for op_idx, op in enumerate(ops):
        for o in op.outputs:
            shape = o.get_shape()
            shape = [s.value for s in shape]
            new_shape = []
            for j, s in enumerate(shape):
                if s == 1 and j == 0:
                    new_shape.append(None)
                else:
                    new_shape.append(s)
            o._shape = tf.TensorShape(new_shape)
    w = sess.graph.get_operation_by_name("softmax/logits/MatMul").inputs[1]
    logits = tf.matmul(tf.squeeze(pool3), w)
    softmax = tf.nn.softmax(logits)

if softmax is None:
  _init_inception()



# @Author: guanwanxian
# @Date:   1970-01-01T08:00:00+08:00
# @Email:  guanwanxian@zju.edu.cn
# @Last modified by:   guanwanxian
# @Last modified time: 2017-02-25T21:13:32+08:00

import numpy as np

def rotMatrix2d(theta):
    c, s = np.cos(theta/180*np.pi), np.sin(theta/180*np.pi)
    M = np.matrix([[c, -s, 0],[s, c, 0],[0, 0, 1]]) # '{} {}; {} {};'.format(c, -s, s, c)
    return M

def transMatrix2d(tx, ty):
    M = np.matrix([[1,0,tx],[0,1,ty],[0,0,1]])
    return M

def transMatrix2dCompose(theta, tx, ty):
    m_r = rotMatrix2d(theta)
    m_t = transMatrix2d(tx, ty)
    return np.matmul(m_t, m_r)

def appendMatrix2d(m, m_add):
    return np.matmul(m, m_add)

def cosAngle(x):
    return np.cos(x/180*np.pi)

def sinAngle(x):
    return np.sin(x/180*np.pi)





def front_expend_step(tiff_file, threshold=1):
    str = (
        (0, 0, 1),
        (0, 0, -1),
        (0, 1, 0),
        (0, -1, 0),
        (1, 0, 0),
        (-1, 0, 0),
    )
    for i in range(tiff_file.shape[0]):
        for j in range(tiff_file.shape[1]):
            for k in range(tiff_file.shape[2]):
                print(i*512**2 + j*512 + k)
                if tiff_file[i][j][k] < threshold:
                    continue
                for d in range(6):
                    dx, dy, dz = i + str[d][0], j + str[d][1], k + str[d][2]
                    if 0<=dx<tiff_file.shape[0] and \
                        0<=dy<tiff_file.shape[1] and \
                        0<=dz<tiff_file.shape[2]:
                        tiff_file[dx][dy][dz] = max(tiff_file[i][j][k], threshold)



import queue

from anytree import NodeMixin
from pyneval.model.swc_node import SwcNode

_3D = "3d"
_2D = "2d"
TRAJECTORY_NONE = -1.0
FLOAT_INF = 999999999.9
DEBUG = False
RIGHT = "right"
LEFT = "left"
DEFULT = "DEFULT"


class BinaryNode(NodeMixin):
    """
        this is a class that temporarily store binarytree transformed from swcTree
        Attributes:
        id: id of the node,
        type: leaf = 1,continuation = 2, bifurcation = 3,
        parent: pa node id,
        son=[]: son list,
        x: x coordinate,
        y: y coordinate,
        z: z coordinate,
        radius: radius of the node
    """

    def __init__(self, data=None, parent=None, left_son=None, right_son=None, max_dep=0, hight=0, treesize=1, leaves=1):
        self.data = data if data is not None else SwcNode()
        self.parent = parent
        self.left_son = left_son
        self.right_son = right_son
        self.max_dep = max_dep
        self.hight = hight
        self.treesize = treesize

    def has_children(self):
        return self.left_son is not None or self.right_son is not None

    def is_left(self):
        return self.get_side() == LEFT

    def get_side(self):
        if self.is_root():
            return DEFULT
        if self.parent.left_son == self:
            return LEFT
        if self.parent.right_son == self:
            return RIGHT
        return LEFT

    def is_root(self):
        return self.parent == None

    def is_leaf(self):
        return self.left_son == None and self.right_son == None

    def to_str(self):
        print (
            "id = {} path_length= {} xy_path_length = {} z_path_length = {}".format(
                self.data.get_id(), self.data.path_length, self.data.xy_path_length, self.data.z_path_length
            )
        )
        if self.left_son is not None:
            print ("left son: {}".format(self.left_son.data.get_id()))
        if self.right_son is not None:
            print ("right son {}".format(self.right_son.data.get_id()))
        print ("---------------------------")

    def print_tree(self):
        q = queue.LifoQueue()
        q.put(self)
        while not q.empty():
            cur = q.get()
            if cur is None:
                continue
            cur.to_str()
            q.put(cur.left_son)
            q.put(cur.right_son)

    def get_node_list(self):
        node_list = list()
        stack = queue.LifoQueue()
        stack.put(self)

        while not stack.empty():
            node = stack.get()
            node_list.append(node)
            if node.has_children():
                stack.put(node.left_son)
                stack.put(node.right_son)
        return node_list



# @Author: guanwanxian
# @Date:   1970-01-01T08:00:00+08:00
# @Email:  guanwanxian@zju.edu.cn
# @Last modified by:   guanwanxian
# @Last modified time: 2017-11-14T20:47:21+08:00

"""ImageUtils, providing basic functions for drawing synthesized microscopic images
"""

from enum import Enum
import numpy as np


def normalizeImage(im):
    im = np.asarray(im, np.float)
    im = np.where(im>255, 255, im)
    im = np.where(im<0, 0, im)
    #im = (im-im.min())*255.0/(im.max()-im.min())
    #im = im.astype(np.uint8)
    return im

def bboxCheck3D(z,y,x,R,img_shape):
    '''bboxCheck: Chech box inside image.
    Args:
        x (float): x pos
        y (float): y pos
        z (float): z pos
        R (int): box radius
        img_shape ((int,int,int)): image shape
    Returns:
        (bool).
    '''
    return (z-R>=0 and z+R<=img_shape[0] and y-R>=0 and y+R<=img_shape[1] and x-R>=0 and x+R<=img_shape[2])

def bboxCheck3DSeparate(z,y,x,rz,ry,rx,img_shape):
    '''bboxCheck: Chech box inside image.
    Args:
        x (float): x pos
        y (float): y pos
        z (float): z pos
        R (int): box radius
        img_shape ((int,int,int)): image shape
    Returns:
        (bool).
    '''
    return (z-rz>=0 and z+rz<=img_shape[0] and y-ry>=0 and y+ry<=img_shape[1] and x-rx>=0 and x+rx<=img_shape[2])

class ImgType(Enum):
    NonBranch = 1
    Branch = 2

class OpType(Enum):

    GetIndex = 1

    GenerateImage = 2

    GetBlackIndex = 3

    GenerageBlackImage = 4

def noisyAddGray(noise_typ,image, op_type):
    """
    Parameters
    ----------
    image : ndarray
        Input image data. Will be converted to float.
    mode : str
        One of the following strings, selecting the type of noise to add:

        'gauss'     Gaussian-distributed additive noise.
        'poisson'   Poisson-distributed noise generated from the data.
        's&p'       Replaces random pixels with 0 or 1.
        'speckle'   Multiplicative noise using out = image + n*image,where
                    n is uniform noise with specified mean & variance.
    """
    if noise_typ == "gauss":
        row,col= image.shape
        mean = 0
        if op_type == OpType.GenerateImage:
            var = 0.1
        elif op_type == OpType.GenerageBlackImage:
            var = 0.1 * np.random.randint(3,6)
        else:
            print('Wrong type, add gauss noise')

        sigma = var**0.5
        gauss = np.random.normal(mean,sigma,(row,col))
        gauss = gauss.reshape(row,col)
        noisy = image + gauss
        return noisy

    elif noise_typ == "s&p":
        avg_intensity = np.asarray(image).mean()
        row,col = image.shape
        s_vs_p = 0.5
        amount = 0.05
        out = image.copy()
        # Salt mode
        num_salt = np.ceil(amount * image.size * s_vs_p)
        coords = [np.random.randint(0, i - 1, int(num_salt))
              for i in image.shape]

        if op_type == OpType.GenerateImage:
            out[coords] = np.where(out[coords]*1.1>255, 255, out[coords]*1.1)
        elif op_type == OpType.GenerageBlackImage:
            out[coords] = np.where(out[coords]*1.5>255, 255, out[coords]*1.5)
        else:
            print('Wrong type, add s&p noise')


        # Pepper mode
        num_pepper = np.ceil(amount* image.size * (1. - s_vs_p))
        coords = [np.random.randint(0, i - 1, int(num_pepper))
              for i in image.shape]
        if op_type == OpType.GenerateImage:
            out[coords] = np.where(out[coords]>avg_intensity, out[coords]*0.8, out[coords]*0.1)
        elif op_type == OpType.GenerageBlackImage:
            out[coords] = np.where(out[coords]>avg_intensity, out[coords]*0.6, out[coords]*0.1)
        else:
            print('Wrong type, add s&p noise')

        return out

    elif noise_typ == "poisson":
        vals = len(np.unique(image))
        vals = 2 ** np.ceil(np.log2(vals))
        noisy = np.random.poisson(image * vals) / float(vals)
        return noisy

    elif noise_typ =="speckle":
        row,col = image.shape
        gauss = np.random.randn(row,col)
        gauss = gauss.reshape(row,col)

        if op_type == OpType.GenerateImage:
            grade = np.random.randint(1,6)
        elif op_type == OpType.GenerageBlackImage:
            grade = np.random.randint(6,20)
        else:
            print('Wrong type, add speckle noise')

        noisy = image + image * gauss * 0.01 * grade
        return noisy

def gaussianNoisyAddGray3D(image,mean,std):
    row,col,dep= image.shape
    gauss = np.random.normal(mean,std,(row,col,dep))
    gauss = gauss.reshape(row,col,dep)
    noisy = image + gauss
    noisy = normalizeImage(noisy)
    return noisy

def noisyAddGray3D(noise_typ,image):
    """
    Parameters
    ----------
    image : ndarray
        Input image data. Will be converted to float.
    mode : str
        One of the following strings, selecting the type of noise to add:

        'gauss'     Gaussian-distributed additive noise.
        'poisson'   Poisson-distributed noise generated from the data.
        's&p'       Replaces random pixels with 0 or 1.
        'speckle'   Multiplicative noise using out = image + n*image,where
                    n is uniform noise with specified mean & variance.
    """
    if noise_typ == "gauss":
        row,col,dep= image.shape
        mean = 40
        sigma = 2#np.random.randint(10,20)
        gauss = np.random.normal(mean,sigma,(row,col,dep))
        gauss = gauss.reshape(row,col,dep)
        noisy = image + gauss
        # 标准化到0-255
        noisy = normalizeImage(noisy)
        return noisy
    elif noise_typ == "poisson":
        noisy = np.random.poisson(lam=image,size=None)
        # 标准化到0-255
        noisy = normalizeImage(noisy)
        return noisy
    elif noise_typ == "s&p":
        avg_intensity = np.asarray(image).mean()
        row,col = image.shape
        s_vs_p = 0.5
        amount = 0.05
        out = image.copy()
        # Salt mode
        num_salt = np.ceil(amount * image.size * s_vs_p)
        coords = [np.random.randint(0, i - 1, int(num_salt))
              for i in image.shape]

        if op_type == OpType.GenerateImage:
            out[coords] = np.where(out[coords]*1.1>255, 255, out[coords]*1.1)
        elif op_type == OpType.GenerageBlackImage:
            out[coords] = np.where(out[coords]*1.5>255, 255, out[coords]*1.5)
        else:
            print('Wrong type, add s&p noise')


        # Pepper mode
        num_pepper = np.ceil(amount* image.size * (1. - s_vs_p))
        coords = [np.random.randint(0, i - 1, int(num_pepper))
              for i in image.shape]
        if op_type == OpType.GenerateImage:
            out[coords] = np.where(out[coords]>avg_intensity, out[coords]*0.8, out[coords]*0.1)
        elif op_type == OpType.GenerageBlackImage:
            out[coords] = np.where(out[coords]>avg_intensity, out[coords]*0.6, out[coords]*0.1)
        else:
            print('Wrong type, add s&p noise')

        return out
    elif noise_typ =="speckle":
        row,col = image.shape
        gauss = np.random.randn(row,col)
        gauss = gauss.reshape(row,col)

        if op_type == OpType.GenerateImage:
            grade = np.random.randint(1,6)
        elif op_type == OpType.GenerageBlackImage:
            grade = np.random.randint(6,20)
        else:
            print('Wrong type, add speckle noise')

        noisy = image + image * gauss * 0.01 * grade
        return noisy



import numpy as np
import scipy.misc
import time

def make_generator(path, n_files, batch_size):
    epoch_count = [1]
    def get_epoch():
        images = np.zeros((batch_size, 3, 64, 64), dtype='int32')
        files = range(n_files)
        random_state = np.random.RandomState(epoch_count[0])
        random_state.shuffle(files)
        epoch_count[0] += 1
        for n, i in enumerate(files):
            image = scipy.misc.imread("{}/{}.png".format(path, str(i+1).zfill(len(str(n_files)))))
            images[n % batch_size] = image.transpose(2,0,1)
            if n > 0 and n % batch_size == 0:
                yield (images,)
    return get_epoch

def load(batch_size, data_dir='/home/ishaan/data/imagenet64'):
    return (
        make_generator(data_dir+'/train_64x64', 1281149, batch_size),
        make_generator(data_dir+'/valid_64x64', 49999, batch_size)
    )

if __name__ == '__main__':
    train_gen, valid_gen = load(64)
    t0 = time.time()
    for i, batch in enumerate(train_gen(), start=1):
        print "{}\t{}".format(str(time.time() - t0), batch[0][0,0,0,0])
        if i == 1000:
            break
        t0 = time.time()


# -*- coding: utf-8 -*-


"""Utility module
   This utility module provides some fundamental functions.
"""
import numpy as np
import math
import os
import sys
import shutil
from shutil import copyfile
from shutil import move
import pickle
import platform
import json
import time
from email import encoders
from email.header import Header
from email.mime.text import MIMEText
from email.utils import parseaddr, formataddr
import smtplib

def is_number(s):
    try:
        float(s)
        return True
    except ValueError:
        pass
 
    try:
        import unicodedata
        unicodedata.numeric(s)
        return True
    except (TypeError, ValueError):
        pass
 
    return False

def dict2npy(data, out_path):
    assert(type(data)==dict)
    np.save(out_path, data)

def npy2dict(in_path):
    data = np.load(in_path).item()
    assert(type(data)==dict)
    return data

def dict2json(data, out_path):
    assert(type(data)==dict)
    with open(out_path, 'w') as fp:
        json.dump(data, fp)

def json2dict(in_path):
    with open(in_path, 'r') as fp:
        return json.load(fp)

def list2dict(list_data):
    return dict((val,idx) for idx,val in enumerate(list_data))

def reverseDict(dic_data):
    return {v:k for k,v in dic_data.items()}


class Timer():
    """ Timer util
    """
    def __init__(self):
        self.init_time = time.time()
        self.last_time = self.init_time
        self.now_time = self.init_time

    def __str__(self):
        return 'init time:{}, last time:{}'.format(self.init_time, self.last_time)

    def reset(self):
        self.init_time = time.time()
        self.last_time = self.init_time
        self.now_time = self.init_time

    def getTime(self):
        self.now_time = time.time()
        duration = self.now_time - self.last_time
        self.last_time = self.now_time
        return duration

    def getTotalTime(self,update_last_time=False):
        self.now_time = time.time()
        duration = self.now_time - self.init_time
        if update_last_time:
            self.last_time = self.now_time
        return duration

    def convertSecondsToStr(self, s):
        return time.strftime("%H小时:%M分:%S秒", time.gmtime(s))


class Mail(Timer):

    def __init__(self):
        super().__init__()

    def __str__(self):
        return 'My Mail class'

    def notifyMe(self,
                 subject,
                 content='官万先\r\n3110102860@zju.edu.cn',
                 mailto='3110102860@zju.edu.cn',
                 check_time=True):

        from_addr = "3110102860@zju.edu.cn"
        password = "163617"

        if check_time:
                time_to_last = self.convertSecondsToStr(self.getTime())
                time_to_begin = self.convertSecondsToStr(self.getTotalTime())
                content += '\r\n'
                content += '距离上次时间:'+time_to_last
                content += '\r\n'
                content += '距离开始时间:'+time_to_begin

        msg = MIMEText(content, 'plain', 'utf-8')
        msg['From'] = '官万先<%s>' % from_addr
        msg['To'] = mailto
        msg['Subject'] = Header(subject, 'utf-8').encode()

        smtp_server = "smtp.zju.edu.cn"
        server = smtplib.SMTP(smtp_server, 25)
        server.login(from_addr, password)
        server.sendmail(from_addr, [mailto], msg.as_string())
        server.quit()



def revertMat(M):
    return np.linalg.revertMat(M)



def copyFiles(src_dir, dst_dir, count, reset=False):
    if reset:
        resetDir(dst_dir)
    else:
        setDir(dst_dir)

    file_names = os.listdir(src_dir)
    total_cnt = len(file_names)
    idxs = np.arange(total_cnt)
    np.random.shuffle(idxs)

    real_count = np.min([total_cnt, count])
    for i in range(real_count):
        src_path = joinPath(src_dir,file_names[idxs[i]])
        dst_path = joinPath(dst_dir, file_names[idxs[i]])
        copyfile(src_path, dst_path)

    print('文件总数:{}, 拷贝数量:{}'.format(total_cnt, real_count))



def testDir(dir):
    '''Exit the program if dir not exist'''
    print(dir)
    if not os.path.exists(dir):
        sys.exit('Exit: %s not exist!'%(dir))

def setDir(dir):
    '''Create the dir if not exist'''
    if not os.path.exists(dir):
        os.makedirs(dir) # Like `make -p`, recursive create directory if needed

def resetDir(dir):
        shutil.rmtree(dir,ignore_errors=True)
        setDir(dir)

def datestamp():
    return time.strftime("%Y%m%d", time.localtime())

def timestamp():
    return time.strftime("%Y%m%d_%H%M%S", time.localtime())

def hostname():
    return platform.node()

def generate_expid(model_name, extra_comment):
    exp_id = "%s-%s-%s" % (model_name, hostname(), timestamp())
    if extra_comment:
        exp_id = exp_id + '-' + extra_comment
    return exp_id

def saveData(data, path):
    """saveData, save variables into file
       :data: data stored in a list
       :path: string, file patth, ending with .pckl
    """
    f = open(path, 'wb')
    pickle.dump(data, f)
    f.close()

def loadData(path):
    """Load variables from a specified file"""
    f = open(path, 'rb')
    data = pickle.load(f)
    f.close()
    return data


def joinPath(path, *paths):
    return os.path.join(path, *paths)



def prepareTrain(args):
    if args.s : # start training new mode
        extra_comment = None
        if args.e:
            extra_comment = args.e[0]
        exp_id = Utils.generate_expid(model_name, extra_comment)
        exp_dir = '/'.join([FLAGS.all_stuff_dir,exp_id])
        Utils.resetDir(exp_dir)

        log_dir = '/'.join([exp_dir,FLAGS.log_dir])
        Utils.setDir(log_dir)

        checkpoint_dir= '/'.join([exp_dir,FLAGS.checkpoint_dir])
        Utils.setDir(checkpoint_dir)

        specified_checkpoint_dir= '/'.join([exp_dir,FLAGS.specified_checkpoint_dir])
        Utils.setDir(specified_checkpoint_dir)

        retrain = True
        continue_from_pre = False
    elif args.r: # restore and continue training model
        exp_id = args.r[0]
        exp_dir = '/'.join([FLAGS.all_stuff_dir,exp_id])
        Utils.testDir(exp_dir)

        log_dir = '/'.join([exp_dir,FLAGS.log_dir])
        Utils.testDir(log_dir)

        checkpoint_dir= '/'.join([exp_dir,FLAGS.checkpoint_dir])
        Utils.testDir(checkpoint_dir)

        specified_checkpoint_dir= '/'.join([exp_dir,FLAGS.specified_checkpoint_dir])
        Utils.setDir(specified_checkpoint_dir)

        retrain = True
        continue_from_pre = True
    else: # test model
        exp_id = args.t[0]
        exp_dir = '/'.join([FLAGS.all_stuff_dir,exp_id])
        Utils.testDir(exp_dir)

        log_dir = '/'.join([exp_dir,FLAGS.log_dir])
        Utils.testDir(log_dir)

        checkpoint_dir= '/'.join([exp_dir,FLAGS.checkpoint_dir])
        Utils.testDir(checkpoint_dir)

        specified_checkpoint_dir= '/'.join([exp_dir,FLAGS.specified_checkpoint_dir])
        Utils.setDir(specified_checkpoint_dir)

        retrain = False
        continue_from_pre = False



"""
    This is a comprehensive test for PyNeval.
    It includes several basic tests on each metric.
    Every time before merging a branch, committer should run this test and make sure every case is passed.
    If the changes in the commit will change the output, i.e., a debug commit, committer should edit this file synchronously.
"""
import os
import sys

import jsonschema
from pyneval.cli import pyneval
from pyneval.model import swc_node
from pyneval.io import read_json
from pyneval.metric.utils import config_utils
from pyneval.metric.utils import metric_manager
from multiprocessing import Pool, cpu_count

STD_TEST_DATA_PATH = "../../data/std_test_data"
abs_dir = os.path.abspath("../../")
pyneval.import_metrics(abs_dir)


def res_compare(metric_name, file_name, test, std):
    OK_FLAG = True
    for key in test:
        if test[key] != std[key]:
            print("[Wrong Answer]:\nmetric name: {}\nfile name: {}\nindex name: {}\nstd answer: {}\noutput answer: {}".format(
                metric_name, file_name, key, std[key], test[key]
            ))
            OK_FLAG = False
    return OK_FLAG


def std_test(mode=1):
    metric_names = ["ssd", "length", "diadem", "critical_node"]
    ok_num, tot_num = 0, 0
    local_metric_manager = metric_manager.get_metric_manager()

    # for each metric
    for metric_name in metric_names:
        print("[INFO]: Testing Metric: {}".format(metric_name))
        metric_method = local_metric_manager.get_metric_method(metric_name)
        metric_ok_num, metric_tot_num = 0, 0

        # for each file in STD_TEST_DATA_PATH
        # max_procs = cpu_count()
        # results = []
        # p_pool = Pool(max_procs)
        for file_name in os.listdir(STD_TEST_DATA_PATH):
            # print("[INFO]: Testing File: {}".format(file_name))
            # loading the trees
            gold_path = os.path.join(STD_TEST_DATA_PATH, file_name, "gold.swc")
            test_path = os.path.join(STD_TEST_DATA_PATH, file_name, "test.swc")
            gold_tree = swc_node.SwcTree()
            test_tree = swc_node.SwcTree()
            gold_tree.load(gold_path)
            test_tree.load(test_path)
            # loading config
            config = config_utils.get_default_configs(metric_name)
            config_schema = config_utils.get_config_schema(metric_name)
            try:
                jsonschema.validate(config, config_schema)
            except Exception as e:
                raise Exception("[Error: ]Error in analyzing config json file")
            # do evaluation
            res = metric_method(gold_tree, test_tree, config)[0]
            # choice: save metric results or compare with the existing results
            std_path = os.path.join(STD_TEST_DATA_PATH, file_name, "std_{}.json".format(metric_name))
            if mode == 2:
                read_json.save_json(std_path, res)
            else:
                std = read_json.read_json(std_path)
                if res_compare(
                        file_name=file_name,
                        metric_name=metric_name,
                        test=res,
                        std=std
                ):
                    ok_num += 1
                    metric_ok_num += 1
            tot_num += 1
            metric_tot_num += 1
        print("        Passed test case: {}\n"
              "        Total test case: {}".format(metric_ok_num, metric_tot_num))
    print("[INFO]: Overall Result:\n"
          "        Passed test case: {}\n"
          "        Total test case: {}".format(ok_num, tot_num))


if __name__ == "__main__":
    sys.setrecursionlimit(1000000)
    std_test(mode=1)



import unittest
from pyneval.metric.utils.edge_match_utils import is_route_clean, get_edge_rtree, get_idedge_dict
from pyneval.model.swc_node import SwcTree, SwcNode
from pyneval.model.euclidean_point import EuclideanPoint, Line


class TestGetLcaLength(unittest.TestCase):
    test_swc_tree = SwcTree()
    test_rtree = None
    id_edge_dict = None

    def init(self):
        self.test_swc_tree.load("../../data_example/unit_test/get_nearby_edges_tree1.swc")
        self.test_rtree = get_edge_rtree(self.test_swc_tree)
        self.id_edge_dict = get_idedge_dict(self.test_swc_tree)

        self.test_swc_tree.get_lca_preprocess()

    def test_1(self):
        line_tuple_a = tuple([self.test_swc_tree.node_from_id(8),
                              self.test_swc_tree.node_from_id(7)])
        line_tuple_b = tuple([self.test_swc_tree.node_from_id(7),
                              self.test_swc_tree.node_from_id(6)])
        e_node1 = EuclideanPoint(center=[1.03, -2.1, 6.31])
        e_node2 = EuclideanPoint(center=[3.87, 0.98, 1.17])

        # ans = is_route_clean(gold_swc_tree=self.test_swc_tree,
        #                    gold_line_tuple_a=line_tuple_a,
        #                    gold_line_tuple_b=line_tuple_b,
        #                    node1, node2,
        #                    edge_use_dict, vis_list,
        #                    DEBUG):





METRICS = {}
METRICS_ALIAS_MAP = {}

class MetricManager(object):
    _instance = None

    def __new__(cls, *args, **kw):
        if cls._instance is None:
            cls._instance = object.__new__(cls, *args, **kw)
        return cls._instance

    def __init__(self):
        pass

    # function register
    def register(self, name, config, desc="", public=False, alias=None):
        def deco(func):
            metric = {
                'config': config,
                'description': desc,
                'alias': alias if alias is not None else [],
                'method': func,
                'public': public,
            }
            METRICS[name] = metric
            for alia in alias:
                METRICS_ALIAS_MAP[alia] = metric
            return func
        return deco

    def get_root_metric(self, metric):
        if metric in METRICS_ALIAS_MAP:
            return METRICS_ALIAS_MAP[metric]
        elif metric in METRICS:
            return metric

    def get_metric_config(self, metric):
        return METRICS[self.get_root_metric(metric)]

    def get_metric_summary(self, with_description):
        summary = ''
        if with_description:
            for metric in METRICS:
                if METRICS[metric].get('public', False):
                    description = METRICS[metric]['description']
                    summary += '[{}]: '.format(metric) + (description if description else '[No description]') + '\n'
        else:
            summary = ', '.join((filter(lambda m: METRICS[m].get('public', False), METRICS.keys())))

        return summary

    def get_metric_method(self, metric):
        return self.get_metric_config(metric)['method']

# get metric manager
def get_metric_manager():
    return MetricManager()



''' 对基于模型方法生成的图像进行评估
'''

import os
import sys
import numpy as np
import pandas as pd
import cv2
import tifffile
import Utils

def getRawImages(root_path, img_types, npz_folder_path, save_to_npz=False):
    if save_to_npz:
        Utils.resetDir(npz_folder_path)

    standard_files = []
    all_files = []
    # {file_path : image_type}
    types = dict()

    ## 保存所有的图片到字典
    for img_type in img_types:
        dir_path = Utils.joinPath(root_path, img_type)
        file_names = os.listdir(dir_path)
        for file_name in file_names:
            file_path = Utils.joinPath(dir_path, file_name)
            all_files.append(file_path)
            types[file_path] = img_type
            if img_type=='standard':
                standard_files.append(file_path)

            # save image to npz files
            if save_to_npz:
                im = tifffile.imread(file_path)
                np.save(Utils.joinPath(npz_folder_path, file_path.replace('/','-')), im)

    return standard_files, all_files, types


class Matcher():
    def __init__(self):
        pass

    ## 准备相似度计算方法及对应函数
    def funcCreator(self, func_type, *args, **kwargs):
        if func_type=='histogram':
            def func(feature_a, feature_b):
                return cv2.compareHist(np.array(feature_a), np.array(feature_b), kwargs['method'])
            return func
        elif func_type=='moment':
            # 基于颜色矩的相似度计算函数
            def func(feature_a, feature_b):
                ''' calculate similaryty based on moment vector
                    Parameters:
                        feature_a,feature_b: numpy array of shape(4,)
                    Returns:
                        func: function
                '''
                #k_mins = kwargs['k_mins']
                #k_maxs = kwargs['k_maxs']
                #feature_a = (feature_a-k_mins)/(k_maxs-k_mins)
                #feature_b = (feature_b-k_mins)/(k_maxs-k_mins)

                assert(feature_a.shape==(4,) and feature_a.shape==(4,))
                return np.abs(feature_a-feature_b).mean()

            return func
        elif func_type=='signal_to_noise':
            def func(feature_a, feature_b):
                feature_a = feature_a[0]
                feature_b = feature_b[0]
                if feature_a>feature_b:
                    ratio = feature_a/feature_b
                else:
                    ratio = feature_b/feature_a
                #return 2/(1+np.exp(ratio-1))
                return ratio
            return func
        elif func_type=='glcm':
            metrics = kwargs['metrics']
            def func(feature_a, feature_b):
                if metrics=='1_norm_distance':
                    return np.sum(np.abs(feature_a-feature_b))
                elif metrics=='2_norm_distance':
                    # 2_norm_distance
                    return np.sum((feature_a-feature_b)**2)**0.5
                else:
                    sys.exit('error metrics for glcm')
            return func




    def getMethodResult(self, standard_files, img_type_dict, feature_dict, func, bigger_first):
        method_result = dict()
        for im_a_path in standard_files:
            feature_a = feature_dict[im_a_path]
            match_result = list()
            for (im_b_path, feature_b) in feature_dict.items():
                val = func(feature_a, feature_b)
                match_result.append(tuple((im_b_path, img_type_dict[im_b_path], val)))

            # sort
            match_result = sorted(match_result, key=lambda tup:tup[2], reverse=bigger_first)

            # add to dict
            method_result[im_a_path] = match_result

        return method_result

    def calAveragePos(self, match_result, method_name, top):
        method_result = match_result[method_name]
        # create dict: type_sum, type_cnt
        type_sum = dict()
        type_cnt = dict()

        for (im_a_path, match_result) in method_result.items():
            #  create file_stat_sum and file_stat_cnt
            file_stat_sum = dict()
            file_stat_cnt = dict()
            for idx, (im_b_path, im_b_type, dis) in enumerate(match_result[:top]):
                if im_b_type in file_stat_sum:
                    file_stat_sum[im_b_type] += (idx+1)
                    file_stat_cnt[im_b_type] += 1
                else:
                    file_stat_sum[im_b_type] = (idx+1)
                    file_stat_cnt[im_b_type] = 1

            # update type_sum and type_cnt
            for (im_type, avg_pos) in file_stat_sum.items():
                if im_type in type_sum:
                    type_sum[im_type] += file_stat_sum[im_type]/file_stat_cnt[im_type]
                    type_cnt[im_type] += 1
                else:
                    type_sum[im_type] = file_stat_sum[im_type]/file_stat_cnt[im_type]
                    type_cnt[im_type] = 1

        # calculate result
        ret = dict()
        for im_type, sum_of_avgpos in type_sum.items():
            ret[im_type] = type_sum[im_type] / type_cnt[im_type]

        # normalize
        for im_type, avgpos in ret.items():
            ret[im_type] = ret[im_type]/top

        return ret

#     def calculateAll(self):

    def calculatePos(self, img_types, match_result, method_names, tops):
        df = pd.DataFrame()
        for method_name in method_names:
            for top in tops:
                res = self.calAveragePos(match_result, method_name, top)
                series = pd.Series(res, index=img_types)
                col_name = '{}, top {}'.format(method_name, top)
                df[col_name] = series

        return df

    def calculateSimilarity(self,
                            standard_files,
                            types,
                            feature_dict,
                            img_types,
                            methods):
        df = pd.DataFrame()
        similarity_dict = {}
        for method_name, func, bigger_better in methods:
            type_sum = {}
            type_cnt = {}
            type_similarity = {}
            for im_a_path in standard_files:
                im_a_type = types[im_a_path]
                feature_a = feature_dict[im_a_path]
                im_sum = {}
                im_cnt = {}
                for (im_b_path, feature_b) in feature_dict.items():
                    im_b_type = types[im_b_path]
                    similarity = func(feature_a, feature_b)
                    if im_b_type in type_similarity:
                        type_similarity[im_b_type].append(similarity)
                    else:
                        type_similarity[im_b_type] = [similarity]

                    # update im_sum, im_cnt
                    if im_b_type in im_sum:
                        im_sum[im_b_type] += similarity
                        im_cnt[im_b_type] += 1
                    else:
                        im_sum[im_b_type] = similarity
                        im_cnt[im_b_type] = 1

                ## update type_sum, type_cnt
                # calculate averate
                im_result = {}
                for im_type, sum_of_similarity in im_sum.items():
                    im_result[im_type] = im_sum[im_type]/im_cnt[im_type]

                # update
                for im_type, avg_of_similarity in im_result.items():
                    if im_type in type_sum:
                        type_sum[im_type] += im_result[im_type]
                        type_cnt[im_type] += 1
                    else:
                        type_sum[im_type] = im_result[im_type]
                        type_cnt[im_type] = 1

            similarity_dict[method_name] = type_similarity
            series = pd.Series(type_sum, index=img_types)
            col_name = '{}: {}'.format(method_name, '>' if bigger_better else '<')
            df[col_name] = series

        return df, similarity_dict



import numpy as np
import tensorflow as tf

import locale

locale.setlocale(locale.LC_ALL, '')

_params = {}
_param_aliases = {}
def param(name, *args, **kwargs):
    """
    A wrapper for `tf.Variable` which enables parameter sharing in models.

    Creates and returns theano shared variables similarly to `tf.Variable`,
    except if you try to create a param with the same name as a
    previously-created one, `param(...)` will just return the old one instead of
    making a new one.

    This constructor also adds a `param` attribute to the shared variables it 
    creates, so that you can easily search a graph for all params.
    """

    if name not in _params:
        kwargs['name'] = name
        param = tf.Variable(*args, **kwargs)
        param.param = True
        _params[name] = param
    result = _params[name]
    i = 0
    while result in _param_aliases:
        # print 'following alias {}: {} to {}'.format(i, result, _param_aliases[result])
        i += 1
        result = _param_aliases[result]
    return result

def params_with_name(name):
    return [p for n,p in _params.items() if name in n]

def delete_all_params():
    _params.clear()

def alias_params(replace_dict):
    for old,new in replace_dict.items():
        # print "aliasing {} to {}".format(old,new)
        _param_aliases[old] = new

def delete_param_aliases():
    _param_aliases.clear()

# def search(node, critereon):
#     """
#     Traverse the Theano graph starting at `node` and return a list of all nodes
#     which match the `critereon` function. When optimizing a cost function, you
#     can use this to get a list of all of the trainable params in the graph, like
#     so:

#     `lib.search(cost, lambda x: hasattr(x, "param"))`
#     """

#     def _search(node, critereon, visited):
#         if node in visited:
#             return []
#         visited.add(node)

#         results = []
#         if isinstance(node, T.Apply):
#             for inp in node.inputs:
#                 results += _search(inp, critereon, visited)
#         else: # Variable node
#             if critereon(node):
#                 results.append(node)
#             if node.owner is not None:
#                 results += _search(node.owner, critereon, visited)
#         return results

#     return _search(node, critereon, set())

# def print_params_info(params):
#     """Print information about the parameters in the given param set."""

#     params = sorted(params, key=lambda p: p.name)
#     values = [p.get_value(borrow=True) for p in params]
#     shapes = [p.shape for p in values]
#     print "Params for cost:"
#     for param, value, shape in zip(params, values, shapes):
#         print "\t{0} ({1})".format(
#             param.name,
#             ",".join([str(x) for x in shape])
#         )

#     total_param_count = 0
#     for shape in shapes:
#         param_count = 1
#         for dim in shape:
#             param_count *= dim
#         total_param_count += param_count
#     print "Total parameter count: {0}".format(
#         locale.format("%d", total_param_count, grouping=True)
#     )

def print_model_settings(locals_):
    print("Uppercase local vars:")
    all_vars = [(k,v) for (k,v) in locals_.items() if (k.isupper() and k!='T' and k!='SETTINGS' and k!='ALL_SETTINGS')]
    all_vars = sorted(all_vars, key=lambda x: x[0])
    for var_name, var_value in all_vars:
        print("\t{}: {}".format(var_name, var_value))


def print_model_settings_dict(settings):
    print("Settings dict:")
    all_vars = [(k,v) for (k,v) in settings.items()]
    all_vars = sorted(all_vars, key=lambda x: x[0])
    for var_name, var_value in all_vars:
        print("\t{}: {}".format(var_name, var_value))



from pyneval.model.swc_node import SwcNode, SwcTree
from pyneval.io.read_swc import read_swc_trees
import os
import csv


def single_file_analyse(swc_tree):
    size = len(swc_tree.get_node_list())
    blocks = len(swc_tree.root().children)
    return size, blocks


def file_analyse(file_path):
    tree2filename = {}
    swc_trees = read_swc_trees(file_path, tree2filename)
    info = [[],[],[]]
    for swc_tree in swc_trees:
        swc_name = tree2filename[swc_tree]
        size, blocks = single_file_analyse(swc_tree)
        info[0].append(swc_name)
        info[1].append(size)
        info[2].append(blocks)
        print("file_name = {} node num = {} blocks = {}".format(swc_name, size, blocks))
    return info


if __name__ == '__main__':
    info = file_analyse("../../../data/raw")
    with open("../../../data/raw/info.csv", 'w') as f:
        write = csv.writer(f)
        write.writerows(info)




from pyneval.metric.utils import config_utils


def get_simple_lca_length(std_tree, test_gold_dict, node1, node2):
    """
    get the corresponding node of node1 and node2 on std tree.
    calculate the lca distance between them
    Exception:
        Exception("[Error: ] std has not been lca initialized yet")
        std tree need to be initialized before running this function
        example:
            std_tree.get_lca_preprocess()
    """
    if std_tree.depth_array is None:
        raise Exception("[Error: ] std has not been lca initialized yet")
    std_id_node_dict = std_tree.get_id_node_dict()

    tmp_node1 = node1
    tmp_node2 = test_gold_dict[node2]

    if tmp_node2 is None:
        raise Exception("[Error: ]Can not find the corresponding node in std tree. ")

    lca_id = std_tree.get_lca(tmp_node1.get_id(), tmp_node2.get_id())
    if lca_id == -1:
        return config_utils.DINF
    lca_node = std_id_node_dict[lca_id]
    return tmp_node1.root_length + tmp_node2.root_length - 2 * lca_node.root_length


class AugmentPath():
    def __init__(self, n_num, m_num):
        self.n_num = n_num
        self.m_num = m_num
        assert(0 <= self.n_num and 0 <= self.m_num)
        self.pa = [-1] * self.n_num
        self.pb = [-1] * self.m_num
        self.vis = [0] * self.n_num
        self.dfn = 0
        self.res = 0
        self.g = []
        for i in range(self.n_num):
            self.g.append([])

    def add(self, src, to):
        assert (0 <= src < self.n_num and 0 <= to < self.m_num)
        self.g[src].append(to)

    def dfs(self, v):
        self.vis[v] = self.dfn
        for u in self.g[v]:
            if self.pb[u] == -1:
                self.pb[u] = v
                self.pa[v] = u
                return True
        for u in self.g[v]:
            if self.vis[self.pb[u]] != self.dfn and self.dfs(self.pb[u]):
                self.pa[v] = u
                self.pb[u] = v
                return True
        return False

    def solve(self):
        while True:
            self.dfn += 1
            cnt = 0
            for i in range(self.n_num):
                if self.pa[i] == -1 and self.dfs(i):
                    cnt += 1
            if cnt == 0:
                break
            self.res += cnt
        return self.res



# @Author: guanwanxian
# @Date:   1970-01-01T08:00:00+08:00
# @Email:  guanwanxian@zju.edu.cn
# @Last modified by:   guanwanxian
# @Last modified time: 2017-02-25T20:57:35+08:00

"""Draw2DTools, providing functions to generate 2d images
"""

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.lines as lines
import matplotlib.transforms as mtransforms
import matplotlib.text as mtext

from Utils import joinPath
from PlotTools import createNewAxes
from ImageUtils import ImgType, OpType
import Draw2DTools as draw2d

class MyLine(lines.Line2D):
    def __init__(self, *args, **kwargs):
        # we'll update the position when the line data is set
        self.text = mtext.Text(0, 0, '')
        lines.Line2D.__init__(self, *args, **kwargs)

        # we can't access the label attr until *after* the line is
        # inited
        self.text.set_text(self.get_label())

    def set_figure(self, figure):
        self.text.set_figure(figure)
        lines.Line2D.set_figure(self, figure)

    def set_axes(self, axes):
        self.text.set_axes(axes)
        lines.Line2D.set_axes(self, axes)

    def set_transform(self, transform):
        # 2 pixel offset
        texttrans = transform + mtransforms.Affine2D().translate(2, 2)
        self.text.set_transform(texttrans)
        lines.Line2D.set_transform(self, transform)

    def set_data(self, x, y):
        if len(x):
            self.text.set_position((x[-1], y[-1]))

        lines.Line2D.set_data(self, x, y)

    def draw(self, renderer):
        # draw my label at the end of the line with 2 pixel offset
        lines.Line2D.draw(self, renderer)
        self.text.draw(renderer)

def generateLines(img_type):
    if img_type==ImgType.NonBranch:
        neighbor_number = np.random.randint(1,3)  # 1~2

    else:
        neighbor_number = np.random.randint(3,5) # 3~4

    lines_array = []
    center_r = np.random.randint(0, 4) # 1~5
    center_angle = np.random.randint(0, 360)
    x0 = int(center_r * np.cos(np.pi * center_angle/180))
    y0 = int(center_r * np.sin(np.pi * center_angle/180))

    for i in range(neighbor_number):
        neighbor_r = np.random.randint(6, 11) # 6~10
        neighbor_angle = np.random.randint(0, 360)
        x1 = int(neighbor_r * np.cos(np.pi * neighbor_angle/180))
        y1 = int(neighbor_r * np.sin(np.pi * neighbor_angle/180))

        # print x0, y0, x1, y1, neighbor_angle
        lines_array.append(MyLine([x0, x1], [y0, y1], lw=20, c='k'))

    return lines_array

def generateLines2(img_type):

    lines_array = []
    x0 = int(1 * np.cos(np.pi * 0/180))
    y0 = int(1 * np.sin(np.pi * 0/180))

    x1 = int(5 * np.cos(np.pi * 180/180))
    y1 = int(5 * np.sin(np.pi * 180/180))
    lines_array.append(MyLine([x0, x1], [y0, y1], lw=5, c='k'))

    x1 = int(5 * np.cos(np.pi * 270/180))
    y1 = int(5 * np.sin(np.pi * 270/180))
    lines_array.append(MyLine([x0, x1], [y0, y1], lw=5, c='k'))

    x1 = int(5 * np.cos(np.pi * 45/180))
    y1 = int(5 * np.sin(np.pi * 45/180))
    lines_array.append(MyLine([x0, x1], [y0, y1], lw=5, c='k'))

    return lines_array

def generateLines3(img_type):
    if img_type==ImgType.NonBranch:
        neighbor_number = np.random.randint(1,3)  # 1~2

    else:
        neighbor_number = np.random.randint(3,5) # 3~4
    margin = int(360/neighbor_number)
    print('margin: ', margin)

    lines_array = []
    center_r = np.random.randint(0, 4) # 1~5
    center_angle = np.random.randint(0, 360)
    x0 = int(center_r * np.cos(np.pi * center_angle/180))
    y0 = int(center_r * np.sin(np.pi * center_angle/180))

    for i in range(neighbor_number):
        neighbor_r = np.random.randint(6, 11) # 6~10
        neighbor_angle = np.random.randint(0, margin) + i*margin
        x1 = int(neighbor_r * np.cos(np.pi * neighbor_angle/180))
        y1 = int(neighbor_r * np.sin(np.pi * neighbor_angle/180))

        print(x0, y0, x1, y1, neighbor_angle)
        lines_array.append(MyLine([x0, x1], [y0, y1], lw=5, c='k'))

    return lines_array

def drawImage(img_type, dest_path, img_name):
#     fig = plt.figure()
    plt.clf()
    fig = plt.gcf()
    fig.set_size_inches(10, 10)
    ax = plt.gca(xlim=[-10,10],ylim=[-10,10]) # Set x and y range

    lines_array = generateLines(img_type)
    for l in lines_array:
        ax.add_line(l)

    # plt.axis('off')
    plt.tight_layout(pad=0)
    plt.savefig(joinPath(dest_path,img_name), bbox_inches='tight', pad_inches=0, frameon=False, dpi=4)

def getVertexSequence(pa,pb, line_width_range):
    ''' Separate va-vb to more segments, and returen the points list as well as line_widths list
    '''
    segs = np.random.randint(20,30)

    pts = [pa]
    for i in range(1,segs):
        pc = tuple([pa[j]+(pb[j]-pa[j])*i*1.0/segs for j in range(2)])
        pts.append(pc)
    pts.append(pb)

    line_widths = []

    line_segs = [([pts[idx][0], pts[idx+1][0]],[pts[idx][1], pts[idx+1][1]]) for idx in range(segs)]
    line_widths = np.linspace(line_width_range[0], line_width_range[1], segs)
#     print(line_segs)
#     print('line width: ',line_widths)
    return line_segs, line_widths

def drawGradLines(ax, line_segs, line_widths):
    for l,w in zip(line_segs, line_widths):
        ax.add_line(MyLine(l[0], l[1], lw=w, c='k'))

def sliceRange(s, t, n):
    arr, step = np.linspace(s, t, n, endpoint=False, retstep=True)
    arr = [(x+step/3, x+step*2/3) for x in arr]
    return arr

def rotMatrix2d(theta):
    c, s = np.cos(theta/180*np.pi), np.sin(theta/180*np.pi)
    M = np.matrix([[c, -s, 0],[s, c, 0],[0, 0, 1]]) # '{} {}; {} {};'.format(c, -s, s, c)
    return M

def transMatrix2d(tx, ty):
    M = np.matrix([[1,0,tx],[0,1,ty],[0,0,1]])
    return M

def transMatrix2dCompose(theta, tx, ty):
    m_r = rotMatrix2d(theta)
    m_t = transMatrix2d(tx, ty)
    return np.matmul(m_t, m_r)

def appendMatrix2d(m, m_add):
    return np.matmul(m, m_add)

def cosAngle(x):
    return np.cos(x/180*np.pi)

def sinAngle(x):
    return np.sin(x/180*np.pi)

def getRandomThetaAndRadiu(theta_range, r_range):
    return np.random.uniform(theta_range[0],theta_range[1]), np.random.randint(r_range[0], r_range[1])

def grow2(m0, ps, theta, r, ax, line_width_range):
    """grow2, 在局部坐标系第一象限绘制
    """
    x = r*cosAngle(theta)
    y = r*sinAngle(theta)

    p = np.array([[x],[y],[1]])
    p_t = np.matmul(m0, p)
    p_t = tuple(np.asarray(p_t).flatten())
#     print(x, y, '\n', p_s,'\n', p_t)
    # ax.add_line(MyLine([ps[0], p_t[0]], [ps[1], p_t[1]], lw=5, c='k'))
    line_segs, line_widths = getVertexSequence(ps, p_t, line_width_range)
    drawGradLines(ax, line_segs, line_widths)

    m1 = transMatrix2dCompose(theta, x, y)
    m0_new = appendMatrix2d(m0, m1)
    return m0_new, p_t

def expand(revert_matrix, start_point, ax, depth, img_type, max_width):
    if depth>0:
        if img_type==ImgType.Branch:
            branch_numbers = np.random.randint(2,4)
            angle_slices = sliceRange(-90, 90, branch_numbers)
            for angle in angle_slices:
                angle_s = angle[0]
                angle_width = angle[1]-angle[0]

                revert_matrix_local = appendMatrix2d(revert_matrix, rotMatrix2d(angle_s))
                theta_range = (0, angle_width)
                radiu_range = (3*depth+1, 3*depth+9)
                local_max_width = max_width
                local_min_width = max_width-np.random.randint(1, 11)
                line_width_range = (local_max_width, local_min_width)
                # line_width_range = (5*depth+15, np.random.randint(5*depth+10, 5*depth+16))
                theta,radiu = getRandomThetaAndRadiu(theta_range, radiu_range)
                revert_matrix_new, start_point_new = grow2(revert_matrix_local,
                                                           start_point,
                                                           theta,
                                                           radiu,
                                                           ax,
                                                           line_width_range)
                expand(revert_matrix_new, start_point_new, ax, depth-1, img_type, local_min_width)

        else: # NonBranch
            angle_s = -40
            angle_width = 80

            revert_matrix_local = appendMatrix2d(revert_matrix, rotMatrix2d(angle_s))
            theta_range = (0, angle_width)
            radiu_range = (8, 14)
            line_width_range = (max_width, np.random.randint(max_width-5,max_width+1))
            theta,radiu = getRandomThetaAndRadiu(theta_range, radiu_range)
            revert_matrix_new, start_point_new = grow2(revert_matrix_local,
                                                       start_point,
                                                       theta,
                                                       radiu,
                                                       ax,
                                                       line_width_range)

def initCanvas():
    fig, ax = createNewAxes(1, 1)

    fig.set_size_inches(10, 10)
    ax.set_xlim(-10,10)
    ax.set_ylim(-10,10)

    return fig,ax



def drawWithGrowing(ax, img_type, dest_path, img_name):
    ax.cla()

    if img_type==ImgType.Branch:
        start_point = (-10, -10, 1)
        revert_matrix = draw2d.transMatrix2d(-10,-10)

        max_width = np.random.randint(30, 131)
        min_width = max_width - np.random.randint(1, 11)
        line_width_range = (max_width, min_width)
        radiu_range = (10,14)
        theta_range = (30,60)
        theta,radiu = draw2d.getRandomThetaAndRadiu(theta_range, radiu_range)
        revert_matrix, start_point = draw2d.grow2(revert_matrix, start_point, theta, radiu, ax, line_width_range)

        expand_depth = 1 + (np.random.uniform()>0.8);
        draw2d.expand(revert_matrix, start_point, ax, expand_depth, img_type, min_width)
    else:
        start_point = (-10, -10, 1)
        revert_matrix = draw2d.transMatrix2d(-10,-10)

        max_width = np.random.randint(100, 131)
        min_width = np.random.randint(max_width-5,max_width+1)
        line_width_range = (max_width, min_width)
        radiu_range = (13,17)
        theta_range = (30,60)
        theta,radiu = draw2d.getRandomThetaAndRadiu(theta_range, radiu_range)
        revert_matrix, start_point = draw2d.grow2(revert_matrix, start_point, theta, radiu, ax, line_width_range)

        draw2d.expand(revert_matrix, start_point, ax, 1, img_type, min_width)

    plt.axis('off')
    plt.tight_layout(pad=0)
    plt.savefig(joinPath(dest_path, img_name), pad_inches=0, frameon=False, dpi=4)
    # plt.savefig(joinPath(dest_path, img_name), bbox_inches='tight', pad_inches=0, frameon=False)
